from abqpy.decorators import abaqus_class_doc, abaqus_method_doc
from .DiagnosticPrint import DiagnosticPrint
from .Monitor import Monitor
from .Restart import Restart
from ..Step.StepBase import StepBase
from ..UtilityAndView.abaqusConstants import *


@abaqus_class_doc
class OutputStep(StepBase):
    """The Step object stores the parameters that determine the context of the step. The Step
    object is the abstract base type for other Step objects. The Step object has no explicit
    constructor. The methods and members of the Step object are common to all objects
    derived from the Step.

    .. note:: 
        This object can be accessed by::

            import step
            mdb.models[name].steps[name]
    """

    @abaqus_method_doc
    def DiagnosticPrint(
        self,
        allke: Boolean = ON,
        criticalElement: Boolean = ON,
        dmass: Boolean = OFF,
        etotal: Boolean = OFF,
        contact: Boolean = ON,
        modelChange: Boolean = OFF,
        plasticity: Boolean = OFF,
        residual: Boolean = ON,
        frequency: int = 1,
        solve: Boolean = ON,
        mass: Boolean = OFF,
    ) -> DiagnosticPrint:
        """This method creates a DiagnosticPrint object.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].steps[name].DiagnosticPrint

        Parameters
        ----------
        allke
            A Boolean specifying a request for a column containing the total kinetic energy. This
            argument is valid only for an Abaqus/Explicit analysis. The default value is ON.
        criticalElement
            A Boolean specifying a request for a column containing the element that has the smallest
            stable time increment and a column listing the value. This argument is valid only for an
            Abaqus/Explicit analysis. The default value is ON.
        dmass
            A Boolean specifying a request for a column containing the percent change in total mass
            of the model as a result of mass scaling. This argument is valid only for an
            Abaqus/Explicit analysis. The default value is OFF unless mass scaling is present in the
            model.
        etotal
            A Boolean specifying a request for a column containing the energy balance of the model.
            This argument is valid only for an Abaqus/Explicit analysis. The default value is OFF.
        contact
            A Boolean specifying a request for detailed output of points that are contacting or
            separating in interface and gap problems. This argument is valid only for an
            Abaqus/Standard analysis. The default value is ON.
        modelChange
            A Boolean specifying a request for detailed output of which elements are being removed
            or reactivated in the step. This argument is valid only for an Abaqus/Standard analysis.
            The default value is OFF.
        plasticity
            A Boolean specifying a request for detailed output of element and integration point
            numbers for which the plasticity algorithms have failed to converge in the material
            routines. This argument is valid only for an Abaqus/Standard analysis. The default value
            is OFF.
        residual
            A Boolean specifying a request for output of equilibrium residuals during the
            equilibrium iterations. This argument is valid only for an Abaqus/Standard analysis. The
            default value is ON.
        frequency
            An Int specifying the frequency of output, in increments. The default value is 1.
        solve
            A Boolean specifying a request for information regarding the actual number of equations
            and the wavefront in each iteration. This argument is valid only for an Abaqus/Standard
            analysis. The default value is ON.
        mass
            A Boolean specifying a request for a column containing the total mass of the model as a
            result of mass scaling. This argument is valid only for an Abaqus/Explicit analysis. The
            default value is OFF.

        Returns
        -------
        diagnosticPrint: DiagnosticPrint
            A :py:class:`~abaqus.StepOutput.DiagnosticPrint.DiagnosticPrint` object
        """
        self.diagnosticPrint = diagnosticPrint = DiagnosticPrint(
            allke,
            criticalElement,
            dmass,
            etotal,
            contact,
            modelChange,
            plasticity,
            residual,
            frequency,
            solve,
            mass,
        )
        return diagnosticPrint

    @abaqus_method_doc
    def Monitor(self, node: str, dof: SymbolicConstant, frequency: int) -> Monitor:
        """This method creates a request for a degree of freedom to be monitored in a general or
        modal procedure.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].steps[name].Monitor

        Parameters
        ----------
        node
            A String specifying the name of the region to be monitored.
        dof
            A SymbolicConstant specifying the degree of freedom to be monitored at the node.
            Possible values are:
            
            - U1
            - U2
            - U3
            - UR1
            - UR2
            - UR3
            - WARP
            - FLUID_PRESSURE
            - ELECTRICAL_POTENTIAL
            - NT11
            - NT30
            - NN11
            - NN30
            
            The NT identifiers are not available for mass diffusion. The NN identifiers are
            available only for mass diffusion.
        frequency
            An Int specifying the output frequency in increments. This argument is valid only for an
            Abaqus/Standard analysis.

        Returns
        -------
        monitor: Monitor
            A :py:class:`~abaqus.StepOutput.Monitor.Monitor` object
        """
        self.monitor = monitor = Monitor(node, dof, frequency)
        return monitor

    @abaqus_method_doc
    def Restart(
        self,
        numberIntervals: int = 0,
        timeMarks: Boolean = OFF,
        overlay: Boolean = OFF,
        frequency: int = 0,
    ) -> Restart:
        """This method creates a restart request.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].steps[name].Restart

        Parameters
        ----------
        numberIntervals
            An Int specifying the number of intervals during the step at which restart information
            will be written. The default value is 0. The default value is 1.
        timeMarks
            A Boolean specifying whether to use exact time marks for writing during an analysis. The
            default value is OFF. The default value is OFF.
        overlay
            A Boolean specifying that only one increment per step should be retained on the restart
            file, thus minimizing the size of the restart file. The default value is OFF. The
            default value is ON.
        frequency
            An Int specifying the increments at which restart information will be written. The
            default value is 0. The default value is 0.This argument applies only to Abaqus/Standard
            analyses.

        Returns
        -------
        restart: Restart
            A :py:class:`~abaqus.StepOutput.Restart.Restart` object

        Raises
        ------
        RangeError
        """
        self.restart = restart = Restart(numberIntervals, timeMarks, overlay, frequency)
        return restart



from __future__ import annotations
from typing import Optional, Dict, overload
from typing_extensions import Literal

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc
from .HistoryOutput import HistoryOutput
from .HistoryPoint import HistoryPoint
from ..UtilityAndView.abaqusConstants import *


@abaqus_class_doc
class HistoryRegion:
    """The HistoryRegion object contains history data for a single location in the model.

    .. note:: 
        This object can be accessed by::

            import odbAccess
            session.odbs[name].steps[name].historyRegions[name]
    """

    #: A SymbolicConstant specifying the position of the history output. Possible values are
    #: NODAL, INTEGRATION_POINT, WHOLE_ELEMENT, WHOLE_REGION, and WHOLE_MODEL.
    position: Optional[SymbolicConstant] = None

    #: A repository of HistoryOutput objects.
    historyOutputs: Dict[str, HistoryOutput] = {}

    #: A String specifying the name of the HistoryRegion object.
    name: str

    #: A String specifying the description of the HistoryRegion object.
    description: str

    #: A :py:class:`~abaqus.Odb.HistoryPoint.HistoryPoint` object specifying the point to which the history data refer.
    point: HistoryPoint

    #: None or an OdbLoadCase object specifying the load case associated with the HistoryRegion
    #: object. The default value is None.
    loadCase: Optional[str] = None

    @abaqus_method_doc
    def __init__(
        self,
        name: str,
        description: str,
        point: HistoryPoint,
        loadCase: Optional[str] = None,
    ) -> None:
        """This method creates a HistoryRegion object.

        .. note:: 
            This function can be accessed by::

                session.odbs[name].steps[name].HistoryRegion

        Parameters
        ----------
        name
            A String specifying the name of the HistoryRegion object.
        description
            A String specifying the description of the HistoryRegion object.
        point
            A :py:class:`~abaqus.Odb.HistoryPoint.HistoryPoint` object specifying the point to which the history data refer.
        loadCase
            None or an OdbLoadCase object specifying the load case associated with the HistoryRegion
            object. The default value is None.

        Returns
        -------
        HistoryRegion
            A :py:class:`~abaqus.Odb.HistoryRegion.HistoryRegion` object.
        """
        ...

    @overload
    @abaqus_method_doc
    def getSubset(self, variableName: str) -> HistoryRegion:
        """This method returns a subset of the data in the HistoryRegion object.

        Parameters
        ----------
        variableName
            A String specifying the name of the output variable to return.

        Returns
        -------
        HistoryRegion
            A :py:class:`~abaqus.Odb.HistoryRegion.HistoryRegion` object.
        """
        ...

    @overload
    @abaqus_method_doc
    def getSubset(self, start: float) -> HistoryRegion:
        """This method returns a subset of the data in the HistoryRegion object.

        Parameters
        ----------
        start
            A Float specifying the start of the subset. This is the same as the first item in the
            data array member of the HistoryOutput object.

        Returns
        -------
        HistoryRegion
            A :py:class:`~abaqus.Odb.HistoryRegion.HistoryRegion` object.
        """
        ...

    @overload
    @abaqus_method_doc
    def getSubset(self, start: float, end: float) -> HistoryRegion:
        """This method returns a subset of the data in the HistoryRegion object.

        Parameters
        ----------
        start
            A Float specifying the start of the subset. This is the same as the first item in the
            data array member of the HistoryOutput object.
        end
            A Float specifying the end of the subset.

        Returns
        -------
        HistoryRegion
            A :py:class:`~abaqus.Odb.HistoryRegion.HistoryRegion` object.
        """
        ...

    @abaqus_method_doc
    def getSubset(self, *args, **kwargs) -> HistoryRegion:
        ...

    def HistoryOutput(
        self,
        name: str,
        description: str,
        type: Literal[SCALAR],
        validInvariants: Optional[SymbolicConstant] = None,
    ) -> HistoryOutput:
        """This method creates a HistoryOutput object.

        .. note:: 
            This function can be accessed by::

                session.odbs[name].steps[name].HistoryRegion

        Parameters
        ----------
        name
            A String specifying the output variable name.
        description
            A String specifying the output variable.
        type
            A SymbolicConstant specifying the output type. Only SCALAR is currently supported.
        validInvariants
            A sequence of SymbolicConstants specifying which invariants should be calculated for
            this field. Possible values are MAGNITUDE, MISES, TRESCA, PRESS, INV3, MAX_PRINCIPAL,
            MID_PRINCIPAL, and MIN_PRINCIPAL. The default value is an empty sequence.

        Returns
        -------
        HistoryOutput
            A :py:class:`~abaqus.Odb.HistoryOutput.HistoryOutput` object.
        """
        self.historyOutputs[name] = historyOutput = HistoryOutput(
            name, description, type, validInvariants
        )
        return historyOutput



from abqpy.decorators import abaqus_class_doc, abaqus_method_doc
from .BoundaryCondition import BoundaryCondition
from ..Region.Region import Region
from ..UtilityAndView.abaqusConstants import *


@abaqus_class_doc
class EulerianBC(BoundaryCondition):
    """The EulerianBC object stores the data for an Eulerian boundary condition.
    The EulerianBC object is derived from the BoundaryCondition object.

    .. note:: 
        This object can be accessed by::

            import load
            mdb.models[name].boundaryConditions[name]
    """

    #: A String specifying the boundary condition repository key.
    name: str = ""

    #: A SymbolicConstant specifying the flow conditions to be defined. Possible values are
    #: INFLOW, OUTFLOW, and BOTH. The default value is INFLOW.
    definition: SymbolicConstant = INFLOW

    #: A SymbolicConstant specifying the control of material flow into the Eulerian domain.
    #: Possible values are FREE, NONE, and VOID. The default value is FREE.
    inflowType: SymbolicConstant = FREE

    #: A SymbolicConstant specifying the control of flow of material out of the Eulerian
    #: domain. Possible values are ZERO_PRESSURE, FREE, NON_REFLECTING, and EQUILIBRIUM. The
    #: default value is ZERO_PRESSURE.
    outflowType: SymbolicConstant = ZERO_PRESSURE

    #: A SymbolicConstant specifying the category of the boundary condition. Possible values
    #: are MECHANICAL and THERMAL.
    category: SymbolicConstant = None

    #: A :py:class:`~abaqus.Region.Region.Region` object specifying the region to which the boundary condition is applied.
    region: Region = Region()

    #: None or a DatumCsys object specifying the local coordinate system of the boundary
    #: condition's degrees of freedom. If **localCsys** = None, the degrees of freedom are defined
    #: in the global coordinate system. The default value is None.
    localCsys: str = None

    @abaqus_method_doc
    def __init__(
        self,
        name: str,
        createStepName: str,
        region: Region,
        definition: SymbolicConstant = INFLOW,
        inflowType: SymbolicConstant = FREE,
        outflowType: SymbolicConstant = ZERO_PRESSURE,
    ):
        """This method creates a EulerianBC object.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].EulerianBC

        Parameters
        ----------
        name
            A String specifying the boundary condition repository key.
        createStepName
            A String specifying the name of the step in which the boundary condition is created.
        region
            A :py:class:`~abaqus.Region.Region.Region` object specifying the region to which the boundary condition is applied.
        definition
            A SymbolicConstant specifying the flow conditions to be defined. Possible values are
            INFLOW, OUTFLOW, and BOTH. The default value is INFLOW.
        inflowType
            A SymbolicConstant specifying the control of material flow into the Eulerian domain.
            Possible values are FREE, NONE, and VOID. The default value is FREE.
        outflowType
            A SymbolicConstant specifying the control of flow of material out of the Eulerian
            domain. Possible values are ZERO_PRESSURE, FREE, NON_REFLECTING, and EQUILIBRIUM. The
            default value is ZERO_PRESSURE.

        Returns
        -------
        EulerianBC
            An :py:class:`~abaqus.BoundaryCondition.EulerianBC.EulerianBC` object.
        """
        super().__init__()

    @abaqus_method_doc
    def setValues(
        self,
        region: Region,
        definition: SymbolicConstant = INFLOW,
        inflowType: SymbolicConstant = FREE,
        outflowType: SymbolicConstant = ZERO_PRESSURE,
    ):
        """This method modifies the data for an existing EulerianBC object in the step where it is
        created.

        Parameters
        ----------
        region
            A :py:class:`~abaqus.Region.Region.Region` object specifying the region to which the boundary condition is applied.
        definition
            A SymbolicConstant specifying the material flow conditions to be defined. Possible
            values are INFLOW, OUTFLOW, and BOTH. The default value is INFLOW.
        inflowType
            A SymbolicConstant specifying the control of material flow into the Eulerian domain.
            Possible values are FREE, NONE, and VOID. The default value is FREE.
        outflowType
            A SymbolicConstant specifying the control of material flow out of the Eulerian domain.
            Possible values are ZERO_PRESSURE, FREE, NON_REFLECTING, and EQUILIBRIUM. The default
            value is ZERO_PRESSURE.
        """
        ...

    @abaqus_method_doc
    def setValuesInStep(
        self,
        stepName: str,
        definition: SymbolicConstant = INFLOW,
        inflowType: SymbolicConstant = FREE,
        outflowType: SymbolicConstant = ZERO_PRESSURE,
    ):
        """This method modifies the propagating data for an existing EulerianBC object in the
        specified step.

        Parameters
        ----------
        stepName
            A String specifying the name of the step in which the boundary condition is modified.
        definition
            A SymbolicConstant specifying the material flow conditions to be defined. Possible
            values are INFLOW, OUTFLOW, and BOTH. The default value is INFLOW.
        inflowType
            A SymbolicConstant specifying the control of material flow into the Eulerian domain.
            Possible values are FREE, NONE, and VOID. The default value is FREE.
        outflowType
            A SymbolicConstant specifying the control of material flow out of the Eulerian domain.
            Possible values are ZERO_PRESSURE, FREE, NON_REFLECTING, and EQUILIBRIUM. The default
            value is ZERO_PRESSURE.
        """
        ...



from abqpy.decorators import abaqus_class_doc
from .LoadState import LoadState
from ..UtilityAndView.abaqusConstants import *


@abaqus_class_doc
class SurfaceTractionState(LoadState):
    """The SurfaceTractionState object stores the propagating data for a SurfaceTraction object
    in a step. One instance of this object is created internally by the SurfaceTraction
    object for each step. The instance is also deleted internally by the SurfaceTraction
    object.
    The SurfaceTractionState object has no constructor or methods.
    The SurfaceTractionState object is derived from the LoadState object.

    .. note:: 
        This object can be accessed by::

            import load
            mdb.models[name].steps[name].loadStates[name]

        The corresponding analysis keywords are:

        - DSLOAD
        - DLOAD
    """

    #: A Float or a Complex specifying the load magnitude.
    magnitude: float = None

    #: A SymbolicConstant specifying the propagation state of the load magnitude. Possible
    #: values are UNSET, SET, UNCHANGED, and MODIFIED.
    magnitudeState: SymbolicConstant = None

    #: A SymbolicConstant specifying the propagation state of the **amplitude** member. Possible
    #: values are UNSET, SET, UNCHANGED, and FREED.
    amplitudeState: SymbolicConstant = None

    #: A SymbolicConstant specifying the propagation state of the LoadState object. Possible
    #: values are:
    #: 
    #: - NOT_YET_ACTIVE
    #: - CREATED
    #: - PROPAGATED
    #: - MODIFIED
    #: - DEACTIVATED
    #: - NO_LONGER_ACTIVE
    #: - TYPE_NOT_APPLICABLE
    #: - INSTANCE_NOT_APPLICABLE
    #: - BUILT_INTO_BASE_STATE
    status: SymbolicConstant = None

    #: A String specifying the name of the amplitude reference. The String is empty if the load
    #: has no amplitude reference.
    amplitude: str = ""



from abqpy.decorators import abaqus_class_doc, abaqus_method_doc
from ..UtilityAndView.abaqusConstants import *


@abaqus_class_doc
class AnalyticSurfaceSegment:
    """An individual segment of the analytic surface.

    .. note:: 
        This object can be accessed by::

            import odbAccess
            session.odbs[name].parts[name].analyticSurface.segments[i]
            session.odbs[name].rootAssembly.instances[name].analyticSurface.segments[i]
            session.odbs[name].steps[name].frames[i].fieldOutputs[name].values[i].instance.analyticSurface.segments[i]
    """

    #: A SymbolicConstant specifying the type of AnalyticSurfaceSegment. Possible values are
    #: START, LINE, CIRCLE, and PARABOLA.
    type: SymbolicConstant

    #: A sequence of sequences of Floats specifying the coordinates of point/s representing the
    #: segment of the AnalyticSurface object. If **type** = CIRCLE, the first row contains
    #: coordinates of the end point and the second row contains coordinates of the center
    #: point. If **type** = PARABOLA, the first row contains coordinates of the middle point and
    #: the second row contains coordinates of the end point. If **type** = START or **type** = LINE, a
    #: single row contains coordinates of the start/end point.
    data: tuple

    @abaqus_method_doc
    def __init__(self, type: SymbolicConstant, data: tuple):
        """This method creates an AnalyticSurfaceSegment object.

        .. note:: 
            This function can be accessed by::

                odbAccess.AnalyticSurfaceSegment

        Parameters
        ----------
        type
            A SymbolicConstant specifying the type of AnalyticSurfaceSegment. Possible values are
            START, LINE, CIRCLE, and PARABOLA.
        data
            A sequence of sequences of Floats specifying the coordinates of point/s representing the
            segment of the AnalyticSurface object. If **type** = CIRCLE, the first row contains
            coordinates of the end point and the second row contains coordinates of the center
            point. If **type** = PARABOLA, the first row contains coordinates of the middle point and
            the second row contains coordinates of the end point. If **type** = START or **type** = LINE, a
            single row contains coordinates of the start/end point.

        Returns
        -------
        AnalyticSurfaceSegment
            An :py:class:`~abaqus.Odb.AnalyticSurfaceSegment.AnalyticSurfaceSegment` object.
        """
        ...



from abqpy.decorators import abaqus_class_doc, abaqus_method_doc
from ..UtilityAndView.abaqusConstants import *


@abaqus_class_doc
class CaeGuiPrefs:
    """The CaeGuiPrefs object contains the details of the graphical preferences in a
    guiPreferences section of the abaqus_2021.gpr file.

    .. note:: 
        This object can be accessed by::

            import caePrefsAccess
            caePrefsAccess.openGuiPreferences(...)
    """

    #: A String specifying the path to the preferences file.
    fileName:str = ""

    @abaqus_method_doc
    def save(self, backupFile: Boolean = True):
        """This method saves the guiPreferences section specified in the current **fileName**.

        Parameters
        ----------
        backupFile: Boolean
            A Boolean specifying whether Abaqus should save a numbered backup copy of the
            preferences file, **fileName**. Default is True.
        """
        # TODO: Implement this method
        ...

    @abaqus_method_doc
    def saveAs(self, fileName: str):
        """This method saves the guiPreferences settings to the specified location.

        Parameters
        ----------
        fileName: str
            A String specifying the path to the preferences file.
        """
        # TODO: Implement this method
        ...



from abqpy.decorators import abaqus_class_doc, abaqus_method_doc
from .Load import Load
from ..Region.Region import Region
from ..UtilityAndView.abaqusConstants import *


@abaqus_class_doc
class SurfaceCurrentDensity(Load):
    """The SurfaceCurrentDensity object stores the data for a surface current.
    The SurfaceCurrentDensity object is derived from the Load object.

    .. note:: 
        This object can be accessed by::

            import load
            mdb.models[name].loads[name]
    """

    #: A String specifying the load repository key.
    name: str = ""

    #: A SymbolicConstant specifying how the load is distributed spatially. Possible values are
    #: UNIFORM and USER_DEFINED. The default value is UNIFORM.
    distributionType: SymbolicConstant = UNIFORM

    #: A :py:class:`~abaqus.Region.Region.Region` object specifying the region to which the load is applied.
    region: Region = Region()

    @abaqus_method_doc
    def __init__(
        self,
        name: str,
        createStepName: str,
        region: Region,
        comp1: str,
        comp2: str,
        comp3: str,
        distributionType: SymbolicConstant = UNIFORM,
        amplitude: str = UNSET,
    ):
        """This method creates a SurfaceCurrentDensity object.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].SurfaceCurrentDensity

        Parameters
        ----------
        name
            A String specifying the load repository key.
        createStepName
            A String specifying the name of the step in which the load is created. This must be the
            first analysis step name.
        region
            A :py:class:`~abaqus.Region.Region.Region` object specifying the region to which the load is applied.
        comp1
            A Complex specifying the first component of the load.
        comp2
            A Complex specifying the second component of the load.
        comp3
            A Complex specifying the third component of the load.
        distributionType
            A SymbolicConstant specifying how the load is distributed spatially. Possible values are
            UNIFORM and USER_DEFINED. The default value is UNIFORM.
        amplitude
            A String or the SymbolicConstant UNSET specifying the name of the amplitude reference.
            UNSET should be used if the load has no amplitude reference. The default value is UNSET.
            You should provide the **amplitude** argument only if it is valid for the specified step.

        Returns
        -------
        SurfaceCurrentDensity
            A :py:class:`~abaqus.Load.SurfaceCurrentDensity.SurfaceCurrentDensity` object.
        """
        super().__init__()

    @abaqus_method_doc
    def setValues(
        self, distributionType: SymbolicConstant = UNIFORM, amplitude: str = UNSET
    ):
        """This method modifies the data for an existing SurfaceCurrentDensity object in the step
        where it is created.

        Parameters
        ----------
        distributionType
            A SymbolicConstant specifying how the load is distributed spatially. Possible values are
            UNIFORM and USER_DEFINED. The default value is UNIFORM.
        amplitude
            A String or the SymbolicConstant UNSET specifying the name of the amplitude reference.
            UNSET should be used if the load has no amplitude reference. The default value is UNSET.
            You should provide the **amplitude** argument only if it is valid for the specified step.
        """
        ...

    @abaqus_method_doc
    def setValuesInStep(
        self,
        stepName: str,
        comp1: str = "",
        comp2: str = "",
        comp3: str = "",
        amplitude: str = "",
    ):
        """This method modifies the propagating data for an existing SurfaceCurrentDensity object
        in the specified step.

        Parameters
        ----------
        stepName
            A String specifying the name of the step in which the load is modified.
        comp1
            A Complex specifying the first component of the load.
        comp2
            A Complex specifying the second component of the load.
        comp3
            A Complex specifying the third component of the load.
        amplitude
            A String or a SymbolicConstant specifying the name of the amplitude reference. Possible
            values for the SymbolicConstant are UNCHANGED and FREED. UNCHANGED should be used if the
            amplitude is propagated from the previous static analysis step. FREED should be used if
            the load is changed to have no amplitude reference. You should provide the **amplitude**
            argument only if it is valid for the specified step.
        """
        ...



from __future__ import annotations
from typing import Optional, Union, Tuple, overload
from typing_extensions import Literal

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc
from ..BasicGeometry.Cell import Cell
from ..BasicGeometry.Edge import Edge
from ..BasicGeometry.Face import Face
from ..BasicGeometry.InterestingPoint import InterestingPoint
from ..BasicGeometry.Vertex import Vertex
from ..BasicGeometry.Transform import Transform
from ..Datum.Datum import Datum
from ..Datum.DatumPlane import DatumPlane
from ..Mesh.MeshFace import MeshFace
from ..Mesh.MeshNode import MeshNode
from ..Sketcher.ConstrainedSketch import ConstrainedSketch
from ..UtilityAndView.abaqusConstants import *


@abaqus_class_doc
class Feature:
    """Abaqus/CAE is a feature-based modeling system, and features are stored in the Feature
    object. The user defines the parameters of the feature, and Abaqus/CAE modifies the
    model based on the value of the parameters. This evaluation of the parameters is called
    regeneration of the feature. Feature objects contain both the parameters and the
    resulting model modification.

    .. note:: 
        This object can be accessed by::

            import part
            mdb.models[name].parts[name].features[name]
            mdb.models[name].parts[name].featuresById[i]
            import assembly
            mdb.models[name].rootAssembly.features[name]
            mdb.models[name].rootAssembly.featuresById[i]
    """

    #: A String specifying the repository key.
    name: str = ""

    #: An Int specifying the ID of the feature.
    id: Optional[int] = None

    @abaqus_method_doc
    def AttachmentPoints(
        self,
        name: str,
        points: float,
        projectionMethod: Literal[
            PROJECT_BY_PROXIMITY, PROJECT_BY_DIRECTION
        ] = PROJECT_BY_PROXIMITY,
        projectOnFaces: Tuple[Face, ...] = (),
        projectOnElementFaces: Tuple[MeshFace, ...] = (),
        projectionDirStartPt: Optional[float] = None,
        projectionDirEndPt: Optional[float] = None,
        setName: str = "",
    ) -> Feature:
        """This method creates an attachment points Feature. Attachment points may be created using
        datum points, vertices, reference points, attachment points, interesting points, orphan
        mesh nodes or coordinates. Optionally, the attachment points can be projected on
        geometric faces or element faces.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].parts[name].AttachmentPoints
                mdb.models[name].rootAssembly.AttachmentPoints

        Parameters
        ----------
        name
            A String specifying a unique Feature name.
        points
            A tuple of points. Each point can be a ConstrainedSketchVertex, Datum point, Reference point, Attachment
            point, orphan mesh Node, Interesting point object, or a tuple of Floats representing the
            coordinates of a point.
        projectionMethod
            A SymbolicConstant specifying the projection method. Possible values are
            PROJECT_BY_PROXIMITY and PROJECT_BY_DIRECTION. The default value is
            PROJECT_BY_PROXIMITY.
        projectOnFaces
            A sequence of Face objects specifying the geometry faces onto which the points are to be
            projected.
        projectOnElementFaces
            A sequence of MeshFace objects specifying the orphan mesh element faces onto which the
            points are to be projected.
        projectionDirStartPt
            A point specifying the start point of the projection direction. The point can be a
            ConstrainedSketchVertex, Datum point, Reference point, Attachment point, orphan mesh Node, Interesting
            point object, or a tuple of Floats representing the coordinates of a point.
        projectionDirEndPt
            A point specifying the end point of the projection direction. The point can be a ConstrainedSketchVertex,
            Datum point, Reference point, Attachment point, orphan mesh Node, Interesting point
            object, or a tuple of Floats representing the coordinates of a point.
        setName
            A String specifying a unique set name.

        Returns
        -------
        feature: Feature
            A :py:class:`~abaqus.Feature.Feature.Feature` object
        """
        ...

    @abaqus_method_doc
    def AttachmentPointsAlongDirection(
        self,
        name: str,
        startPoint: float,
        pointCreationMethod: Literal[AUTO_FIT, NUM_PTS_ALONG_DIR, NUM_PTS_BETWEEN_PTS],
        endPoint: Optional[float] = None,
        direction: str = "",
        spacing: str = "",
        numPtsAlongDir: str = "",
        numPtsBetweenPts: str = "",
        createPtAtStartPt: Boolean = True,
        createPtAtEndPt: Boolean = True,
        projectionMethod: Literal[
            PROJECT_BY_PROXIMITY, PROJECT_BY_DIRECTION
        ] = PROJECT_BY_PROXIMITY,
        projectOnFaces: Tuple[Face, ...] = (),
        projectOnElementFaces: Tuple[MeshFace, ...] = (),
        projectionDirStartPt: Optional[float] = None,
        projectionDirEndPt: Optional[float] = None,
        flipDirection: Boolean = OFF,
        setName: str = "",
    ) -> Feature:
        """This method creates a Feature object by creating attachment points along a direction or
        between two points. A Datum point, a ConstrainedSketchVertex, a Reference point, an Attachment point, an
        Interesting point, or an orphan mesh Node can be specified as the start or end point.
        The direction can be specified using a straight edge or a datum axis.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].parts[name].AttachmentPoints
                mdb.models[name].rootAssembly.AttachmentPoints

        Parameters
        ----------
        name
            A String specifying a unique Feature name.
        startPoint
            A point specifying the start point of the direction along which to create points. The
            point can be a ConstrainedSketchVertex, Datum point, Reference point, Attachment point, orphan mesh Node,
            Interesting point object, or a tuple of Floats representing the coordinates of a point.
        pointCreationMethod
            A SymbolicConstant specifying the point creation method. Possible values are AUTO_FIT,
            NUM_PTS_ALONG_DIR, and NUM_PTS_BETWEEN_PTS.
        endPoint
            A point specifying the end point if creating points between two points. The point can be
            a ConstrainedSketchVertex, Datum point, Reference point, Attachment point, orphan mesh Node, Interesting
            point object, or a tuple of Floats representing the coordinates of a point.
        direction
            The direction can be specified by a straight edge or a datum axis.
        spacing
            A float specifying the spacing to be used between two points.
        numPtsAlongDir
            An integer specifying the number of points to be created along the specified direction.
        numPtsBetweenPts
            An integer specifying the number of points to be created between the start and end
            points.
        createPtAtStartPt
            A Boolean specifying whether to create an attachment point at the start point. The
            default value is True.
        createPtAtEndPt
            A Boolean specifying whether to create an attachment point at the end point. The default
            value is True.
        projectionMethod
            A SymbolicConstant specifying the projection method. Possible values are
            PROJECT_BY_PROXIMITY and PROJECT_BY_DIRECTION. The default value is
            PROJECT_BY_PROXIMITY.
        projectOnFaces
            A sequence of Face objects specifying the geometry faces onto which the points are to be
            projected.
        projectOnElementFaces
            A sequence of MeshFace objects specifying the orphan mesh element faces onto which the
            points are to be projected.
        projectionDirStartPt
            A point specifying the start point of the projection direction. The point can be a
            ConstrainedSketchVertex, Datum point, Reference point, Attachment point, orphan mesh Node, Interesting
            point object, or a tuple of Floats representing the coordinates of a point.
        projectionDirEndPt
            A point specifying the end point of the projection direction. The point can be a ConstrainedSketchVertex,
            Datum point, Reference point, Attachment point, orphan mesh Node, Interesting point
            object, or a tuple of Floats representing the coordinates of a point.
        flipDirection
            A Boolean specifying if the direction along which the attachment points are created
            should be reversed. This argument is valid only when
            **pointCreationMethod** = NUM_PTS_ALONG_DIR.
        setName
            A String specifying a unique set name.

        Returns
        -------
        feature: Feature
            A :py:class:`~abaqus.Feature.Feature.Feature` object
        """
        ...

    @abaqus_method_doc
    def AttachmentPointsOffsetFromEdges(
        self,
        name: str,
        edges: tuple,
        startPoint: str = "",
        flipDirection: str = "",
        pointCreationMethod: Literal[BY_NUMBER, BY_SPACING] = ...,
        numberOfPoints: str = "",
        spacingBetweenPoints: str = "",
        offsetFromStartPoint: float = 0,
        offsetFromEndPoint: float = 0,
        spacingMethod: Literal[AUTO_FIT_PTS, SPECIFY_NUM_PTS] = AUTO_FIT_PTS,
        patterningMethod: Literal[PATTERN_ORTHOGONALLY, PATTERN_ALONG_DIRECTION] = ...,
        referenceFace: str = "",
        startPointForPatternDirection: Tuple[float, ...] = ...,
        endPointForPatternDirection: Tuple[float, ...] = ...,
        offsetFromEdges: str = "",
        numberOfRows: int = 1,
        spacingBetweenRows: str = "",
        projectionMethod: Literal[
            PROJECT_BY_PROXIMITY, PROJECT_BY_DIRECTION
        ] = PROJECT_BY_PROXIMITY,
        projectOnFaces: Tuple[Face, ...] = (),
        projectOnElementFaces: Tuple[MeshFace, ...] = (),
        projectionDirStartPt: Tuple[float, ...] = ...,
        projectionDirEndPt: Tuple[float, ...] = ...,
        setName: str = "",
    ) -> Feature:
        """This method creates a Feature object by creating attachment points along or offset from
        one or more connected edges.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].parts[name].AttachmentPoints
                mdb.models[name].rootAssembly.AttachmentPoints

        Parameters
        ----------
        name
            A String specifying a unique Feature name.
        edges
            A sequence of connected Edge objects specifying the geometry edges from which to offset
            the points.
        startPoint
            A ConstrainedSketchVertex of the selected edges that specifies the point from which to create points.
            This point can be one of the two end vertices of the connected edges. In case of edges
            forming a closed loop and having multiple vertices, this point can be any one of the
            vertices on the edges.
        flipDirection
            This parameter is required to indicate the direction in which to create the points. This
            parameter is required only in case of edges forming a closed loop.
        pointCreationMethod
            A SymbolicConstant specifying the point creation method. Possible values are BY_NUMBER
            or BY_SPACING.
        numberOfPoints
            An integer specifying the number of points to be created along the selected edges.
        spacingBetweenPoints
            A float specifying the spacing to be used between two points while creating the points
            between the start and end points of the edges.
        offsetFromStartPoint
            A float specifying the distance by which to offset the first point from the start vertex
            of the edge chain. The default value is 0.0.
        offsetFromEndPoint
            A float specifying the distance by which to offset the last point from the end vertex of
            the edge chain. This parameter should be specified only if the point creation method is
            BY_NUMBER. The default value is 0.0.
        spacingMethod
            A SymbolicConstant specifying the spacing method. Possible values are AUTO_FIT_PTS or
            SPECIFY_NUM_PTS. The default value is AUTO_FIT_PTS.
        patterningMethod
            A SymbolicConstant specifying the method to pattern of points. Possible values are
            PATTERN_ORTHOGONALLY or PATTERN_ALONG_DIRECTION.
        referenceFace
            A geometry Face object adjacent to one of the edges from which to offset the points to
            create a pattern of points when the PATTERN_ORTHOGONALLY method is chosen for
            patterning. The face is used to identify the patterning direction. If the number of rows
            is one and the initial offset is zero, the reference face may not be specified.
        startPointForPatternDirection
            A point specifying the start point of the direction along which to create a pattern of
            points when the PATTERN_ALONG_DIRECTION method is chosen for patterning. The point can
            be a ConstrainedSketchVertex, Datum point, Reference point, Attachment point, orphan mesh Node,
            Interesting point object, or a tuple of Floats representing the coordinates of a point.
        endPointForPatternDirection
            A point specifying the end point of the direction along which to create a pattern of
            points when the PATTERN_ALONG_DIRECTION method is chosen for patterning. The point can
            be a ConstrainedSketchVertex, Datum point, Reference point, Attachment point, orphan mesh Node,
            Interesting point object, or a tuple of Floats representing the coordinates of a point.
        offsetFromEdges
            A float specifying the distance by which to offset the first row of points from the
            edges.
        numberOfRows
            An integer specifying the number of rows of points to be created for the pattern. The
            default value is 1.
        spacingBetweenRows
            A float specifying the spacing to be used between two rows while creating a pattern of
            points.
        projectionMethod
            A SymbolicConstant specifying the projection method. Possible values are
            PROJECT_BY_PROXIMITY and PROJECT_BY_DIRECTION. The default value is
            PROJECT_BY_PROXIMITY.
        projectOnFaces
            A sequence of Face objects specifying the geometry faces onto which the points are to be
            projected.
        projectOnElementFaces
            A sequence of MeshFace objects specifying the orphan mesh element faces onto which the
            points are to be projected.
        projectionDirStartPt
            A point specifying the start point of the projection direction. The point can be a
            ConstrainedSketchVertex, Datum point, Reference point, Attachment point, orphan mesh Node, Interesting
            point object, or a tuple of Floats representing the coordinates of a point.
        projectionDirEndPt
            A point specifying the end point of the projection direction. The point can be a ConstrainedSketchVertex,
            Datum point, Reference point, Attachment point, orphan mesh Node, Interesting point
            object, or a tuple of Floats representing the coordinates of a point.
        setName
            A String specifying a unique set name.

        Returns
        -------
        feature: Feature
            A :py:class:`~abaqus.Feature.Feature.Feature` object
        """
        ...

    @abaqus_method_doc
    def DatumAxisByCylFace(self, face: str) -> Feature:
        """This method creates a Feature object and a DatumAxis object along the axis of a cylinder
        or cone.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].parts[name].AttachmentPoints
                mdb.models[name].rootAssembly.AttachmentPoints

        Parameters
        ----------
        face
            A cylindrical or conical Face object.

        Returns
        -------
        Feature
            A :py:class:`~abaqus.Feature.Feature.Feature` object.

        Raises
        ------
        AbaqusException
        """
        ...

    @abaqus_method_doc
    def DatumAxisByNormalToPlane(self, plane: str, point: int) -> Feature:
        """This method creates a Feature object and a DatumAxis object normal to the specified
        plane and passing through the specified point.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].parts[name].AttachmentPoints
                mdb.models[name].rootAssembly.AttachmentPoints

        Parameters
        ----------
        plane
            A planar Face, an ElementFace, or a Datum object representing a datum plane.
        point
            A ConstrainedSketchVertex, an InterestingPoint, a MeshNode, or a Datum object representing a datum point.

        Returns
        -------
        Feature
            A :py:class:`~abaqus.Feature.Feature.Feature` object.

        Raises
        ------
        AbaqusException
        """
        ...

    @abaqus_method_doc
    def DatumAxisByParToEdge(self, edge: str, point: int) -> Feature:
        """This method creates a Feature object and a DatumAxis object parallel to the specified
        edge and passing through the specified point.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].parts[name].AttachmentPoints
                mdb.models[name].rootAssembly.AttachmentPoints

        Parameters
        ----------
        edge
            A straight Edge, an ElementEdge, or a Datum object representing a datum axis.
        point
            A ConstrainedSketchVertex, an InterestingPoint, a MeshNode, or a Datum object representing a datum point.

        Returns
        -------
        Feature
            A :py:class:`~abaqus.Feature.Feature.Feature` object.

        Raises
        ------
        AbaqusException
        """
        ...

    @abaqus_method_doc
    def DatumAxisByPrincipalAxis(
        self, principalAxis: Literal[XAXIS, YAXIS, ZAXIS]
    ) -> Feature:
        """This method creates a Feature object and a DatumAxis object along one of the three
        principal axes.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].parts[name].AttachmentPoints
                mdb.models[name].rootAssembly.AttachmentPoints

        Parameters
        ----------
        principalAxis
            A SymbolicConstant specifying the principal axis. Possible values are XAXIS, YAXIS, and
            ZAXIS.

        Returns
        -------
        Feature
            A :py:class:`~abaqus.Feature.Feature.Feature` object.

        Raises
        ------
        AbaqusException
        """
        ...

    @overload
    @abaqus_method_doc
    def DatumAxisByRotation(self, line: str, axis: str, angle: float) -> Feature:
        """This method creates a Feature object and a DatumAxis object in a three-dimensional model
        by rotating a line about the specified axis through the specified angle.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].parts[name].AttachmentPoints
                mdb.models[name].rootAssembly.AttachmentPoints

        Parameters
        ----------
        line
            A straight Edge, a Datum object representing a datum axis, or an ElementEdge object
            specifying the line to rotate.
        axis
            A straight Edge, a Datum object representing a datum axis, or an ElementEdge object
            specifying the axis about which to rotate the line.
        angle
            A Float specifying the angle in degrees to rotate the line.

        Returns
        -------
        Feature
            A :py:class:`~abaqus.Feature.Feature.Feature` object.

        Raises
        ------
        AbaqusException
        """
        ...

    @overload
    @abaqus_method_doc
    def DatumAxisByRotation(self, line: str, point: int, angle: float) -> Feature:
        """This method creates a Feature object and a DatumAxis object in a two-dimensional model
        by rotating a line about the specified point through the specified angle.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].parts[name].AttachmentPoints
                mdb.models[name].rootAssembly.AttachmentPoints

        Parameters
        ----------
        line
            A straight Edge, a Datum object representing a datum axis, or an ElementEdge object
            specifying the line to rotate.
        point
            A ConstrainedSketchVertex, an InterestingPoint, a MeshNode, or a Datum object representing a datum point
            specifying the point about which to rotate the line.
        angle
            A Float specifying the angle in degrees to rotate the line.

        Returns
        -------
        Feature
            A :py:class:`~abaqus.Feature.Feature.Feature` object.

        Raises
        ------
        AbaqusException
        """
        ...

    @abaqus_method_doc
    def DatumAxisByRotation(self, *args, **kwargs):
        ...

    def DatumAxisByThreePoint(self, point1: int, point2: int, point3: int) -> Feature:
        """This method creates a Feature object and a DatumAxis object normal to the circle
        described by three points and through its center.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].parts[name].AttachmentPoints
                mdb.models[name].rootAssembly.AttachmentPoints

        Parameters
        ----------
        point1
            A ConstrainedSketchVertex, an InterestingPoint, a MeshNode, or a Datum object representing a datum point
            specifying the first point on the circle.
        point2
            A ConstrainedSketchVertex, an InterestingPoint, a MeshNode, or a Datum object representing a datum point
            specifying the second point on the circle.
        point3
            A ConstrainedSketchVertex, an InterestingPoint, a MeshNode, or a Datum object representing a datum point
            specifying the third point on the circle.

        Returns
        -------
        Feature
            A :py:class:`~abaqus.Feature.Feature.Feature` object.

        Raises
        ------
        AbaqusException
        """
        ...

    @abaqus_method_doc
    def DatumAxisByThruEdge(self, edge: str) -> Feature:
        """This method creates a Feature object and a DatumAxis object along the specified edge.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].parts[name].AttachmentPoints
                mdb.models[name].rootAssembly.AttachmentPoints

        Parameters
        ----------
        edge
            A straight Edge or an ElementEdge object.

        Returns
        -------
        Feature
            A :py:class:`~abaqus.Feature.Feature.Feature` object.

        Raises
        ------
        AbaqusException
        """
        ...

    @abaqus_method_doc
    def DatumAxisByTwoPlane(self, plane1: str, plane2: str) -> Feature:
        """This method creates a Feature object and a DatumAxis object at the intersection of two
        planes.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].parts[name].AttachmentPoints
                mdb.models[name].rootAssembly.AttachmentPoints

        Parameters
        ----------
        plane1
            A planar Face, an ElementFace, or a Datum object representing a datum plane.
        plane2
            A planar Face, an ElementFace, or a Datum object representing a datum plane.

        Returns
        -------
        Feature
            A :py:class:`~abaqus.Feature.Feature.Feature` object.

        Raises
        ------
        AbaqusException
        """
        ...

    @abaqus_method_doc
    def DatumAxisByTwoPoint(self, point1: int, point2: int) -> Feature:
        """This method creates a Feature object and a DatumAxis object along the line joining two
        points.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].parts[name].AttachmentPoints
                mdb.models[name].rootAssembly.AttachmentPoints

        Parameters
        ----------
        point1
            A ConstrainedSketchVertex, an InterestingPoint, a MeshNode, or a Datum object representing a datum point.
        point2
            A ConstrainedSketchVertex, an InterestingPoint, a MeshNode, or a Datum object representing a datum point.

        Returns
        -------
        Feature
            A :py:class:`~abaqus.Feature.Feature.Feature` object.

        Raises
        ------
        AbaqusException
        """
        ...

    @abaqus_method_doc
    def DatumCsysByDefault(
        self, coordSysType: Literal[CARTESIAN, CYLINDRICAL, SPHERICAL], name: str = ""
    ) -> Feature:
        """This method creates a Feature object and a DatumCsys object from the specified default
        coordinate system at the origin.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].parts[name].AttachmentPoints
                mdb.models[name].rootAssembly.AttachmentPoints

        Parameters
        ----------
        coordSysType
            A SymbolicConstant specifying the default coordinate system to be used. Possible values
            are CARTESIAN, CYLINDRICAL, and SPHERICAL.
        name
            A String specifying the name of the DatumCsys.

        Returns
        -------
        Feature
            A :py:class:`~abaqus.Feature.Feature.Feature` object.

        Raises
        ------
        AbaqusException
        """
        ...

    @abaqus_method_doc
    def DatumCsysByOffset(
        self,
        coordSysType: Literal[CARTESIAN, CYLINDRICAL, SPHERICAL],
        datumCoordSys: Datum,
        vector: tuple,
        point: str,
        name: str = "",
    ) -> Feature:
        """This method creates a Feature object and a DatumCsys object by offsetting the origin of
        an existing datum coordinate system to a specified point.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].parts[name].AttachmentPoints
                mdb.models[name].rootAssembly.AttachmentPoints

        Parameters
        ----------
        coordSysType
            A SymbolicConstant specifying the type of coordinate system. Possible values are
            CARTESIAN, CYLINDRICAL, and SPHERICAL.
        datumCoordSys
            A :py:class:`~abaqus.Datum.Datum.Datum` object representing a datum coordinate system from which to offset.
        vector
            A sequence of three Floats specifying the **X**-, **Y**-, and **Z**-offsets from
            **datumCoordSys**. The arguments **vector** and **point** are mutually exclusive, and one of
            them must be specified.
        point
            A ConstrainedSketchVertex, InterestingPoint, DatumPoint object or a sequence of three Floats specifying
            the **X**-, **Y**-, and **Z**-coordinates of a point in space. The point represents the origin
            of the new datum coordinate system. The arguments **vector** and **point** are mutually
            exclusive, and one of them must be specified.
        name
            A String specifying the name of the DatumCsys.

        Returns
        -------
        Feature
            A :py:class:`~abaqus.Feature.Feature.Feature` object.

        Raises
        ------
        AbaqusException
        """
        ...

    @abaqus_method_doc
    def DatumCsysByThreePoints(
        self,
        coordSysType: Literal[CARTESIAN, CYLINDRICAL, SPHERICAL],
        origin: int,
        point1: int,
        point2: int,
        line1: str,
        line2: str,
        name: str = "",
    ) -> Feature:
        """This method creates a Feature object and a DatumCsys object from three points.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].parts[name].AttachmentPoints
                mdb.models[name].rootAssembly.AttachmentPoints

        Parameters
        ----------
        coordSysType
            A SymbolicConstant specifying the type of coordinate system. Possible values are
            CARTESIAN, CYLINDRICAL, and SPHERICAL.
        origin
            A ConstrainedSketchVertex, an InterestingPoint, a MeshNode, or a Datum object representing a datum point
            specifying the origin of the coordinate system.
        point1
            A ConstrainedSketchVertex, an InterestingPoint, a MeshNode, or a Datum object representing a datum point
            specifying a point on the **X**-axis or the rr-axis. The **point1** and **line1** arguments
            are mutually exclusive. One of them must be specified.
        point2
            A ConstrainedSketchVertex, an InterestingPoint, a MeshNode, or a Datum object representing a datum point
            specifying a point in the **X - Y** plane or the rr-θθ plane. The **point2** and **line2**
            arguments are mutually exclusive. One of them must be specified.
        line1
            An Edge, an Element Edge, or a Datum object representing a datum axis specifying the
            **X**-axis or the rr-axis. The **point1** and **line1** arguments are mutually exclusive. One
            of them must be specified.
        line2
            An Edge, an Element Edge, or a Datum object representing a datum axis specifying a
            vector in the **X - Y** plane or the rr-θθ plane. The **point2** and **line2** arguments are
            mutually exclusive. One of them must be specified.
        name
            A String specifying the name of the DatumCsys.

        Returns
        -------
        Feature
            A :py:class:`~abaqus.Feature.Feature.Feature` object.

        Raises
        ------
        AbaqusException
        """
        ...

    @abaqus_method_doc
    def DatumCsysByTwoLines(
        self,
        coordSysType: Literal[CARTESIAN, CYLINDRICAL, SPHERICAL],
        line1: str,
        line2: str,
        name: str = "",
    ) -> Feature:
        """This method creates a Feature object and a DatumCsys object from two orthogonal lines.
        The origin of the new datum coordinate system is placed at the intersection of the two
        lines.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].parts[name].AttachmentPoints
                mdb.models[name].rootAssembly.AttachmentPoints

        Parameters
        ----------
        coordSysType
            A SymbolicConstant specifying the type of coordinate system. Possible values are
            CARTESIAN, CYLINDRICAL, and SPHERICAL.
        line1
            A straight Edge, an ElementEdge, or a Datum object representing a datum axis specifying
            the **X**-axis or the rr-axis.
        line2
            A straight Edge, an ElementEdge, or a Datum object representing a datum axis specifying
            a line in the **X - Y** plane or in the rr-θθ plane.
        name
            A String specifying the name of the DatumCsys.

        Returns
        -------
        Feature
            A :py:class:`~abaqus.Feature.Feature.Feature` object.

        Raises
        ------
        AbaqusException
        """
        ...

    @abaqus_method_doc
    def DatumPlaneByPrincipalPlane(
        self, principalPlane: Literal[XYPLANE, YZPLANE, XZPLANE], offset: float
    ) -> Feature:
        """This method creates a Feature object and a DatumPlane object through the origin along
        one of the three principal planes.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].parts[name].AttachmentPoints
                mdb.models[name].rootAssembly.AttachmentPoints

        Parameters
        ----------
        principalPlane
            A SymbolicConstant specifying the principal plane. Possible values are XYPLANE, YZPLANE,
            and XZPLANE.
        offset
            A Float specifying the offset from the plane.

        Returns
        -------
        Feature
            A :py:class:`~abaqus.Feature.Feature.Feature` object.

        Raises
        ------
        AbaqusException
        """
        ...

    @overload
    @abaqus_method_doc
    def DatumPlaneByOffset(
        self, plane: str, flip: Literal[SIDE1, SIDE2], offset: float
    ) -> Feature:
        """This method creates a Feature object and a DatumPlane object offset by a specified
        distance from an existing plane.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].parts[name].AttachmentPoints
                mdb.models[name].rootAssembly.AttachmentPoints

        Parameters
        ----------
        plane
            A planar Face, an ElementFace, or a Datum object representing a datum plane.
        flip
            A SymbolicConstant specifying whether the normal should be flipped. Possible values are
            SIDE1 and SIDE2.
        offset
            A Float specifying the offset from the plane.

        Returns
        -------
        Feature
            A :py:class:`~abaqus.Feature.Feature.Feature` object.

        Raises
        ------
        AbaqusException
        """
        ...

    @overload
    @abaqus_method_doc
    def DatumPlaneByOffset(self, plane: str, point: int) -> Feature:
        """This method creates a Feature object and a DatumPlane object offset from an existing
        plane and passing through the specified point.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].parts[name].AttachmentPoints
                mdb.models[name].rootAssembly.AttachmentPoints

        Parameters
        ----------
        plane
            A planar Face, an ElementFace, or a Datum object representing a datum plane.
        point
            A ConstrainedSketchVertex, an InterestingPoint, a MeshNode, or a Datum object representing a datum point.

        Returns
        -------
        Feature
            A :py:class:`~abaqus.Feature.Feature.Feature` object.

        Raises
        ------
        AbaqusException
        """
        ...

    @abaqus_method_doc
    def DatumPlaneByOffset(self, *args, **kwargs):
        ...

    def DatumPlaneByRotation(self, plane: str, axis: str, angle: float) -> Feature:
        """This method creates a Feature object and a DatumPlane object by rotating a plane about
        the specified axis through the specified angle.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].parts[name].AttachmentPoints
                mdb.models[name].rootAssembly.AttachmentPoints

        Parameters
        ----------
        plane
            A planar Face, an ElementFace, or a Datum object representing a datum plane.
        axis
            A straight Edge, an ElementEdge, or a Datum object representing a datum axis.
        angle
            A Float specifying the angle in degrees to rotate the plane.

        Returns
        -------
        Feature
            A :py:class:`~abaqus.Feature.Feature.Feature` object.

        Raises
        ------
        AbaqusException
        """
        ...

    @abaqus_method_doc
    def DatumPlaneByThreePoints(self, point1: int, point2: int, point3: int) -> Feature:
        """This method creates a Feature object and a DatumPlane object defined by passing through
        three points.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].parts[name].AttachmentPoints
                mdb.models[name].rootAssembly.AttachmentPoints

        Parameters
        ----------
        point1
            A ConstrainedSketchVertex, an InterestingPoint, a MeshNode, or a Datum object representing a datum point.
        point2
            A ConstrainedSketchVertex, an InterestingPoint, a MeshNode, or a Datum object representing a datum point.
        point3
            A ConstrainedSketchVertex, an InterestingPoint, a MeshNode, or a Datum object representing a datum point.

        Returns
        -------
        Feature
            A :py:class:`~abaqus.Feature.Feature.Feature` object.

        Raises
        ------
        AbaqusException
        """
        ...

    @abaqus_method_doc
    def DatumPlaneByLinePoint(self, line: str, point: int) -> Feature:
        """This method creates a Feature object and a DatumPlane object that pass through the
        specified line and through the specified point that does not lie on the line.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].parts[name].AttachmentPoints
                mdb.models[name].rootAssembly.AttachmentPoints

        Parameters
        ----------
        line
            A straight Edge, an ElementEdge, or a Datum object representing a datum axis.
        point
            A ConstrainedSketchVertex, an InterestingPoint, a MeshNode, or a Datum object representing a datum point.

        Returns
        -------
        Feature
            A :py:class:`~abaqus.Feature.Feature.Feature` object.

        Raises
        ------
        AbaqusException
        """
        ...

    @abaqus_method_doc
    def DatumPlaneByPointNormal(self, point: int, normal: str) -> Feature:
        """This method creates a Feature object and a DatumPlane object normal to the specified
        line and running through the specified point.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].parts[name].AttachmentPoints
                mdb.models[name].rootAssembly.AttachmentPoints

        Parameters
        ----------
        point
            A ConstrainedSketchVertex, an InterestingPoint, a MeshNode, or a Datum object representing a datum point.
        normal
            A straight Edge, an ElementEdge, or a Datum object representing a datum axis.

        Returns
        -------
        Feature
            A :py:class:`~abaqus.Feature.Feature.Feature` object.

        Raises
        ------
        AbaqusException
        """
        ...

    @abaqus_method_doc
    def DatumPlaneByTwoPoint(self, point1: int, point2: int) -> Feature:
        """This method creates a Feature object and a DatumPlane object midway between two points
        and normal to the line connecting the points.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].parts[name].AttachmentPoints
                mdb.models[name].rootAssembly.AttachmentPoints

        Parameters
        ----------
        point1
            A ConstrainedSketchVertex, an InterestingPoint, a MeshNode, or a Datum object representing a datum point.
        point2
            A ConstrainedSketchVertex, an InterestingPoint, a MeshNode, or a Datum object representing a datum point.

        Returns
        -------
        Feature
            A :py:class:`~abaqus.Feature.Feature.Feature` object.

        Raises
        ------
        AbaqusException
        """
        ...

    @abaqus_method_doc
    def DatumPointByCoordinate(self, coords: tuple) -> Feature:
        """This method creates a Feature object and a DatumPoint object at the point defined by the
        specified coordinates.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].parts[name].AttachmentPoints
                mdb.models[name].rootAssembly.AttachmentPoints

        Parameters
        ----------
        coords
            A sequence of three Floats specifying the **X**-, **Y**-, and **Z**-coordinates of the datum
            point.

        Returns
        -------
        feature: Feature
            A :py:class:`~abaqus.Feature.Feature.Feature` object
        """
        ...

    @abaqus_method_doc
    def DatumPointByOffset(self, point: int, vector: tuple) -> Feature:
        """This method creates a Feature object and a DatumPoint object offset from an existing
        point by a vector.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].parts[name].AttachmentPoints
                mdb.models[name].rootAssembly.AttachmentPoints

        Parameters
        ----------
        point
            A ConstrainedSketchVertex, an InterestingPoint, a MeshNode, or a Datum object representing a datum point.
        vector
            A sequence of three Floats specifying the **X**-, **Y**-, and **Z**-offsets from **point**.

        Returns
        -------
        feature: Feature
            A :py:class:`~abaqus.Feature.Feature.Feature` object
        """
        ...

    @abaqus_method_doc
    def DatumPointByMidPoint(self, point1: int, point2: int) -> Feature:
        """This method creates a Feature object and a DatumPoint object midway between two points.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].parts[name].AttachmentPoints
                mdb.models[name].rootAssembly.AttachmentPoints

        Parameters
        ----------
        point1
            A ConstrainedSketchVertex, an InterestingPoint, a MeshNode, or a Datum object representing a datum point.
        point2
            A ConstrainedSketchVertex, an InterestingPoint, a MeshNode, or a Datum object representing a datum point.

        Returns
        -------
        feature: Feature
            A :py:class:`~abaqus.Feature.Feature.Feature` object
        """
        ...

    @abaqus_method_doc
    def DatumPointByOnFace(
        self, face: str, edge1: str, offset1: float, edge2: str, offset2: float
    ) -> Feature:
        """This method creates a Feature object and a DatumPoint object on the specified face,
        offset from two edges.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].parts[name].AttachmentPoints
                mdb.models[name].rootAssembly.AttachmentPoints

        Parameters
        ----------
        face
            A planar Face or a Datum object representing a datum plane.
        edge1
            A straight Edge or a Datum object representing a datum axis.
        offset1
            A Float specifying the offset from **edge1**.
        edge2
            A straight Edge or a Datum object representing a datum axis.
        offset2
            A Float specifying the offset from **edge2**.

        Returns
        -------
        Feature
            A :py:class:`~abaqus.Feature.Feature.Feature` object.

        Raises
        ------
        AbaqusException
        """
        ...

    @abaqus_method_doc
    def DatumPointByEdgeParam(self, edge: Edge, parameter: float) -> Feature:
        """This method creates a Feature object and a DatumPoint object along an edge at a selected
        distance from one end of the edge.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].parts[name].AttachmentPoints
                mdb.models[name].rootAssembly.AttachmentPoints

        Parameters
        ----------
        edge
            An :py:class:`~abaqus.BasicGeometry.Edge.Edge` object.
        parameter
            A Float specifying the distance along **edge** to the DatumPoint object. Possible values
            are 0 << **parameter** << 1.

        Returns
        -------
        Feature
            A :py:class:`~abaqus.Feature.Feature.Feature` object.

        Raises
        ------
        AbaqusException
        RangeError
        """
        ...

    @abaqus_method_doc
    def DatumPointByProjOnEdge(self, point: int, edge: str) -> Feature:
        """This method creates a Feature object and a DatumPoint object along an edge by projecting
        an existing point along the normal to the edge.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].parts[name].AttachmentPoints
                mdb.models[name].rootAssembly.AttachmentPoints

        Parameters
        ----------
        point
            A ConstrainedSketchVertex, an InterestingPoint, a MeshNode, or a Datum object representing a datum point.
        edge
            An Edge, an ElementEdge or a Datum object representing a datum axis.

        Returns
        -------
        Feature
            A :py:class:`~abaqus.Feature.Feature.Feature` object.

        Raises
        ------
        AbaqusException
        """
        ...

    @abaqus_method_doc
    def DatumPointByProjOnFace(self, point: int, face: Face) -> Feature:
        """This method creates a Feature object and a DatumPoint object on a specified face by
        projecting an existing point onto the face.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].parts[name].AttachmentPoints
                mdb.models[name].rootAssembly.AttachmentPoints

        Parameters
        ----------
        point
            A ConstrainedSketchVertex, an InterestingPoint, a MeshNode, or a Datum object representing a datum point.
        face
            A :py:class:`~abaqus.BasicGeometry.Face.Face` object or a Datum object representing a datum plane.Note:Any other types of
            planes are not supported.

        Returns
        -------
        Feature
            A :py:class:`~abaqus.Feature.Feature.Feature` object.

        Raises
        ------
        AbaqusException
        """
        ...

    @abaqus_method_doc
    def MakeSketchTransform(
        self,
        sketchPlane: str,
        origin: tuple = (),
        sketchOrientation: Literal[RIGHT, LEFT, TOP, BOTTOM] = RIGHT,
        sketchPlaneSide: Literal[SIDE1, SIDE2] = SIDE1,
        sketchUpEdge: str = "",
    ) -> Transform:
        """This method creates a Transform object. A :py:class:`~abaqus.BasicGeometry.Transform.Transform` object is a 4x3 matrix of Floats
        that represents the transformation from sketch coordinates to part coordinates.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].parts[name].AttachmentPoints
                mdb.models[name].rootAssembly.AttachmentPoints

        Parameters
        ----------
        sketchPlane
            A Datum plane object or a planar Face object specifying the sketch plane.
        origin
            A sequence of Floats specifying the **X**-, **Y**-, and **Z**-coordinates that will be used as
            the origin of the sketch. The default value is computed as the centroid of the face.
        sketchOrientation
            A SymbolicConstant specifying the orientation of **sketchUpEdge** on the sketch. Possible
            values are RIGHT, LEFT, TOP, and BOTTOM. The default value is RIGHT.
        sketchPlaneSide
            A SymbolicConstant specifying on which side of the **sketchPlane** the sketch is
            positioned. Possible values are SIDE1 and SIDE2. The default value is SIDE1.
        sketchUpEdge
            An Edge or DatumAxis object specifying the orientation of the sketch. If unspecified,
            the sketch is assumed to be oriented with the **Y**-direction pointing up.

        Returns
        -------
        Transform
            A :py:class:`~abaqus.BasicGeometry.Transform.Transform` object. A Transform is an object with one method that returns the transform
        matrix.

        Raises
        ------
        Up direction is parallel to plane normal
            If the sketchUpEdge is parallel to the sketchPlane.
        """
        ...

    @abaqus_method_doc
    def PartitionCellByDatumPlane(
        self, cells: Tuple[Cell, ...], datumPlane: DatumPlane
    ) -> Feature:
        """This method partitions one or more cells using the given datum plane.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].parts[name].AttachmentPoints
                mdb.models[name].rootAssembly.AttachmentPoints

        Parameters
        ----------
        cells
            A sequence of Cell objects specifying the cells to partition.
        datumPlane
            A :py:class:`~abaqus.Datum.DatumPlane.DatumPlane` object.

        Returns
        -------
        Feature
            A :py:class:`~abaqus.Feature.Feature.Feature` object.

        Raises
        ------
        AbaqusException
        """
        ...

    @abaqus_method_doc
    def PartitionCellByExtendFace(self, cells: Tuple[Cell, ...], extendFace: str) -> Feature:
        """This method partitions one or more cells by extending the underlying geometry of a given
        face to partition the target cells.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].parts[name].AttachmentPoints
                mdb.models[name].rootAssembly.AttachmentPoints

        Parameters
        ----------
        cells
            A sequence of Cell objects specifying the cells to partition.
        extendFace
            A planar, cylindrical, conical, or spherical Face object.

        Returns
        -------
        Feature
            A :py:class:`~abaqus.Feature.Feature.Feature` object.

        Raises
        ------
        AbaqusException
        """
        ...

    @abaqus_method_doc
    def PartitionCellByExtrudeEdge(
        self, cells: Tuple[Cell, ...], edges: str, line: str, sense: SymbolicConstant
    ) -> Feature:
        """This method partitions one or more cells by extruding selected edges in the given
        direction.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].parts[name].AttachmentPoints
                mdb.models[name].rootAssembly.AttachmentPoints

        Parameters
        ----------
        cells
            A sequence of Cell objects specifying the cells to partition.
        edges
            The Edge objects to be extruded. The edges must be in the same plane. The edges must
            form a continuous chain, without branches. The edges must belong to the same
            PartInstance object.
        line
            A straight Edge or DatumAxis object specifying the extrude direction. **line** must be
            perpendicular to the plane formed by **edges**.
        sense
            A SymbolicConstant specifying the direction of the extrusion. Possible values are
            FORWARD and REVERSE. If **sense** = FORWARD, the extrusion is in the direction of **line**.

        Returns
        -------
        Feature
            A :py:class:`~abaqus.Feature.Feature.Feature` object.

        Raises
        ------
        AbaqusException
        """
        ...

    @abaqus_method_doc
    def PartitionCellByPatchNCorners(self, cell: Cell, cornerPoints: tuple) -> Feature:
        """This method partitions a cell using an N-sided cutting patch defined by the given corner
        points.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].parts[name].AttachmentPoints
                mdb.models[name].rootAssembly.AttachmentPoints

        Parameters
        ----------
        cell
            A :py:class:`~abaqus.BasicGeometry.Cell.Cell` object specifying the cell to partition.
        cornerPoints
            A sequence of ConstrainedSketchVertex, InterestingPoint, or DatumPoint objects. 3 ≤ len(*cornerPoints*)
            ≤ 5. The corner points must not coincide.

        Returns
        -------
        Feature
            A :py:class:`~abaqus.Feature.Feature.Feature` object.

        Raises
        ------
        AbaqusException
        """
        ...

    @abaqus_method_doc
    def PartitionCellByPatchNEdges(self, cell: str, edges: Tuple[Edge, ...]) -> Feature:
        """This method partitions a cell using an N-sided cutting patch defined by the given edges.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].parts[name].AttachmentPoints
                mdb.models[name].rootAssembly.AttachmentPoints

        Parameters
        ----------
        cell
            A Cell specifying the cell to partition.
        edges
            A sequence of Edge objects bounding the patch. The edges must form a closed loop. The
            Edge objects must belong to the same PartInstance object as **cell**.

        Returns
        -------
        Feature
            A :py:class:`~abaqus.Feature.Feature.Feature` object.

        Raises
        ------
        AbaqusException
        """
        ...

    @abaqus_method_doc
    def PartitionCellByPlaneNormalToEdge(
        self, cells: Tuple[Cell, ...], edge: Edge, point: int
    ) -> Feature:
        """This method partitions one or more cells using a plane normal to an edge at the given
        edge point.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].parts[name].AttachmentPoints
                mdb.models[name].rootAssembly.AttachmentPoints

        Parameters
        ----------
        cells
            A sequence of Cell objects specifying the cells to partition.
        edge
            An :py:class:`~abaqus.BasicGeometry.Edge.Edge` object specifying the normal to the plane.
        point
            A ConstrainedSketchVertex, InterestingPoint, or DatumPoint object specifying a point on **edge**.

        Returns
        -------
        Feature
            A :py:class:`~abaqus.Feature.Feature.Feature` object.

        Raises
        ------
        AbaqusException
        """
        ...

    @abaqus_method_doc
    def PartitionCellByPlanePointNormal(
        self, cells: Tuple[Cell, ...], point: int, normal: str
    ) -> Feature:
        """This method partitions one or more cells using a plane defined by a point and a normal
        direction.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].parts[name].AttachmentPoints
                mdb.models[name].rootAssembly.AttachmentPoints

        Parameters
        ----------
        cells
            A sequence of Cell objects specifying the cells to partition.
        point
            A ConstrainedSketchVertex, InterestingPoint, or DatumPoint object specifying a point on the plane.
        normal
            A straight Edge or DatumAxis object specifying the normal to the plane.

        Returns
        -------
        Feature
            A :py:class:`~abaqus.Feature.Feature.Feature` object.

        Raises
        ------
        AbaqusException
        """
        ...

    @abaqus_method_doc
    def PartitionCellByPlaneThreePoints(
        self, cells: Tuple[Cell, ...], point1: int, point2: int, point3: int
    ) -> Feature:
        """This method partitions one or more cells using a plane defined by three points.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].parts[name].AttachmentPoints
                mdb.models[name].rootAssembly.AttachmentPoints

        Parameters
        ----------
        cells
            A sequence of Cell objects specifying the cells to partition.
        point1
            A ConstrainedSketchVertex, InterestingPoint, or DatumPoint object specifying a point on the plane.
        point2
            A ConstrainedSketchVertex, InterestingPoint, or DatumPoint object specifying a point on the plane.
        point3
            A ConstrainedSketchVertex, InterestingPoint, or DatumPoint object specifying a point on the
            plane.Note:*point1*, **point2**, and **point3** must not be colinear and must not coincide.

        Returns
        -------
        Feature
            A :py:class:`~abaqus.Feature.Feature.Feature` object.

        Raises
        ------
        AbaqusException
        """
        ...

    @abaqus_method_doc
    def PartitionCellBySweepEdge(
        self, cells: Tuple[Cell, ...], edges: Tuple[Edge, ...], sweepPath: Edge
    ) -> Feature:
        """This method partitions one or more cells by sweeping selected edges along the given
        sweep path.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].parts[name].AttachmentPoints
                mdb.models[name].rootAssembly.AttachmentPoints

        Parameters
        ----------
        cells
            A sequence of Cell objects specifying the cells to partition.
        edges
            A sequence of Edge objects to be swept. The edges must be in the same plane. The edges
            must form a continuous chain without branches. The Edge objects must all belong to the
            same PartInstance object.
        sweepPath
            An :py:class:`~abaqus.BasicGeometry.Edge.Edge` object specifying the sweep path. The start of **sweepPath** must be in the plane
            and perpendicular to the plane formed by **edges**. The sweep path must be planar.

        Returns
        -------
        Feature
            A :py:class:`~abaqus.Feature.Feature.Feature` object.

        Raises
        ------
        AbaqusException
        """
        ...

    @abaqus_method_doc
    def PartitionEdgeByDatumPlane(
        self, edges: Tuple[Edge, ...], datumPlane: DatumPlane
    ) -> Feature:
        """This method partitions an edge where it intersects with a datum plane.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].parts[name].AttachmentPoints
                mdb.models[name].rootAssembly.AttachmentPoints

        Parameters
        ----------
        edges
            A sequence of Edge objects specifying the edges to partition.
        datumPlane
            A :py:class:`~abaqus.Datum.DatumPlane.DatumPlane` object specifying the location of the partition.

        Returns
        -------
        Feature
            A :py:class:`~abaqus.Feature.Feature.Feature` object.

        Raises
        ------
        AbaqusException
        """
        ...

    @abaqus_method_doc
    def PartitionEdgeByParam(self, edges: Tuple[Edge, ...], parameter: float) -> Feature:
        """This method partitions one or more edges at the given normalized edge parameter.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].parts[name].AttachmentPoints
                mdb.models[name].rootAssembly.AttachmentPoints

        Parameters
        ----------
        edges
            A sequence of Edge objects specifying the edges to partition.
        parameter
            A Float specifying the normalized distance along **edge** at which to partition. Possible
            values are 0.0 << **parameter** << 1.0.

        Returns
        -------
        Feature
            A :py:class:`~abaqus.Feature.Feature.Feature` object.

        Raises
        ------
        AbaqusException
        """
        ...

    @abaqus_method_doc
    def PartitionEdgeByPoint(self, edge: Edge, point: int) -> Feature:
        """This method partitions an edge at the given point.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].parts[name].AttachmentPoints
                mdb.models[name].rootAssembly.AttachmentPoints

        Parameters
        ----------
        edge
            An :py:class:`~abaqus.BasicGeometry.Edge.Edge` object specifying the edge to partition.
        point
            An InterestingPoint or DatumPoint object specifying a point on **edge**.

        Returns
        -------
        Feature
            A :py:class:`~abaqus.Feature.Feature.Feature` object.

        Raises
        ------
        AbaqusException
        """
        ...

    @abaqus_method_doc
    def PartitionFaceByAuto(self, face: Face) -> Feature:
        """This method automatically partitions a target face into simple regions that can be
        meshed using a structured meshing technique.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].parts[name].AttachmentPoints
                mdb.models[name].rootAssembly.AttachmentPoints

        Parameters
        ----------
        face
            A :py:class:`~abaqus.BasicGeometry.Face.Face` object specifying the face to partition.

        Returns
        -------
        feature: Feature
            A :py:class:`~abaqus.Feature.Feature.Feature` object
        """
        ...

    @abaqus_method_doc
    def PartitionFaceByCurvedPathEdgeParams(
        self, face: Face, edge1: Edge, parameter1: float, edge2: Edge, parameter2: float
    ) -> Feature:
        """This method partitions a face normal to two edges, using a curved path between the two
        given edge points defined by the normalized edge parameters.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].parts[name].AttachmentPoints
                mdb.models[name].rootAssembly.AttachmentPoints

        Parameters
        ----------
        face
            A :py:class:`~abaqus.BasicGeometry.Face.Face` object specifying the face to partition.
        edge1
            An :py:class:`~abaqus.BasicGeometry.Edge.Edge` object specifying the start of the partition. The edge must belong to **face**.
        parameter1
            A Float specifying the distance along **edge1** at which to partition. Possible values are
            0.0 ≤ **distance1** ≤ 1.0.
        edge2
            An :py:class:`~abaqus.BasicGeometry.Edge.Edge` object specifying the end of the partition. The edge must belong to **face**.
        parameter2
            A Float specifying the distance along **edge2** at which to partition. Possible values are
            0.0 ≤ **distance2** ≤ 1.0.

        Returns
        -------
        Feature
            A :py:class:`~abaqus.Feature.Feature.Feature` object.

        Raises
        ------
        AbaqusException
        """
        ...

    @abaqus_method_doc
    def PartitionFaceByCurvedPathEdgePoints(
        self, face: Face, edge1: Edge, point1: int, edge2: Edge, point2: int
    ) -> Feature:
        """This method partitions a face normal to two edges, using a curved path between the two
        given edge points.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].parts[name].AttachmentPoints
                mdb.models[name].rootAssembly.AttachmentPoints

        Parameters
        ----------
        face
            A :py:class:`~abaqus.BasicGeometry.Face.Face` object specifying the face to partition.
        edge1
            An :py:class:`~abaqus.BasicGeometry.Edge.Edge` object specifying the start of the partition. The edge must belong to **face**.
        point1
            A ConstrainedSketchVertex, InterestingPoint, or DatumPoint object specifying a point on **edge1**.
        edge2
            An :py:class:`~abaqus.BasicGeometry.Edge.Edge` object specifying the end of the partition. The edge must belong to **face**.
        point2
            A ConstrainedSketchVertex, InterestingPoint, or DatumPoint object specifying a point on **edge2**.

        Returns
        -------
        Feature
            A :py:class:`~abaqus.Feature.Feature.Feature` object.

        Raises
        ------
        AbaqusException
        """
        ...

    @abaqus_method_doc
    def PartitionFaceByDatumPlane(
        self, faces: Tuple[Face, ...], datumPlane: DatumPlane
    ) -> Feature:
        """This method partitions one or more faces using the given datum plane.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].parts[name].AttachmentPoints
                mdb.models[name].rootAssembly.AttachmentPoints

        Parameters
        ----------
        faces
            A sequence of Face objects specifying the faces to partition.
        datumPlane
            A :py:class:`~abaqus.Datum.DatumPlane.DatumPlane` object specifying the location of the partition.

        Returns
        -------
        feature: Feature
            A :py:class:`~abaqus.Feature.Feature.Feature` object
        """
        ...

    @abaqus_method_doc
    def PartitionFaceByExtendFace(self, faces: Tuple[Face, ...], extendFace: Face) -> Feature:
        """This method partitions one or more faces by extending the underlying geometry of another
        given face to partition the target faces.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].parts[name].AttachmentPoints
                mdb.models[name].rootAssembly.AttachmentPoints

        Parameters
        ----------
        faces
            A sequence of Face objects specifying the faces to partition.
        extendFace
            A :py:class:`~abaqus.BasicGeometry.Face.Face` object that is to be extended to create the partition. The face to extend can be
            a planar, cylindrical, conical, or spherical face.

        Returns
        -------
        feature: Feature
            A :py:class:`~abaqus.Feature.Feature.Feature` object
        """
        ...

    @abaqus_method_doc
    def PartitionFaceByIntersectFace(
        self, faces: Tuple[Face, ...], cuttingFaces: Tuple[Face, ...]
    ) -> Feature:
        """This method partitions one or more faces using the given cutting faces to partition the
        target faces.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].parts[name].AttachmentPoints
                mdb.models[name].rootAssembly.AttachmentPoints

        Parameters
        ----------
        faces
            A sequence of Face objects specifying the faces to partition.
        cuttingFaces
            A sequence of Face objects that specify the cutting faces.

        Returns
        -------
        feature: Feature
            A :py:class:`~abaqus.Feature.Feature.Feature` object
        """
        ...

    @abaqus_method_doc
    def PartitionFaceByProjectingEdges(
        self,
        faces: Tuple[Face, ...],
        edges: Tuple[Edge, ...],
        extendEdges: Boolean = False,
    ) -> Feature:
        """This method partitions one or more faces by projecting the given edges on the target
        faces.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].parts[name].AttachmentPoints
                mdb.models[name].rootAssembly.AttachmentPoints

        Parameters
        ----------
        faces
            A sequence of Face objects specifying the faces to partition.
        edges
            A sequence of Edge objects specifying the edges that will be projected onto the target
            faces.
        extendEdges
            A boolean specifying whether to extend the given edges at their free ends in the tangent
            direction before partitioning the target faces. The default value is False.

        Returns
        -------
        feature: Feature
            A :py:class:`~abaqus.Feature.Feature.Feature` object
        """
        ...

    @abaqus_method_doc
    def PartitionFaceByShortestPath(
        self, faces: Tuple[Face, ...], point1: int, point2: int
    ) -> Feature:
        """This method partitions one or more faces using a minimum distance path between the two
        given points.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].parts[name].AttachmentPoints
                mdb.models[name].rootAssembly.AttachmentPoints

        Parameters
        ----------
        faces
            A sequence of Face objects specifying the face to partition.
        point1
            A ConstrainedSketchVertex, InterestingPoint, or DatumPoint object.
        point2
            A ConstrainedSketchVertex, InterestingPoint, or DatumPoint object.Note:*point1* and **point2** must not
            coincide, and they must both lie on the underlying surface geometry of at least one of
            the target faces.

        Returns
        -------
        Feature
            A :py:class:`~abaqus.Feature.Feature.Feature` object.

        Raises
        ------
        AbaqusException
        """
        ...

    @abaqus_method_doc
    def PartitionFaceBySketch(
        self,
        faces: Tuple[Face, ...],
        sketch: ConstrainedSketch,
        sketchUpEdge: str = "",
        sketchOrientation: Literal[RIGHT, LEFT, TOP, BOTTOM] = RIGHT,
    ) -> Feature:
        """This method partitions one or more planar faces by sketching on them.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].parts[name].AttachmentPoints
                mdb.models[name].rootAssembly.AttachmentPoints

        Parameters
        ----------
        faces
            A sequence of Face objects specifying the faces to partition.
        sketch
            A :py:class:`~abaqus.Sketcher.ConstrainedSketch.ConstrainedSketch` object specifying the partition.
        sketchUpEdge
            An Edge or DatumAxis object specifying the orientation of **sketch**. This edge or datum
            axis must not be orthogonal to the plane defined by **faces**. If unspecified, **sketch** is
            assumed to be oriented in with the **Y** direction pointing up.
        sketchOrientation
            A SymbolicConstant specifying the orientation of **sketchUpEdge** on the sketch. Possible
            values are RIGHT, LEFT, TOP, and BOTTOM. The default value is RIGHT.

        Returns
        -------
        Feature
            A :py:class:`~abaqus.Feature.Feature.Feature` object.

        Raises
        ------
        AbaqusException
        """
        ...

    @abaqus_method_doc
    def PartitionFaceBySketchDistance(
        self,
        faces: Tuple[Face, ...],
        sketchPlane: str,
        sketchPlaneSide: Literal[SIDE1, SIDE2],
        sketchUpEdge: Edge,
        sketch: ConstrainedSketch,
        distance: float,
        sketchOrientation: Literal[RIGHT, LEFT, TOP, BOTTOM] = RIGHT,
    ) -> Feature:
        """This method partitions one or more faces by sketching on a sketch plane and then
        projecting the sketch toward the target faces through the given distance.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].parts[name].AttachmentPoints
                mdb.models[name].rootAssembly.AttachmentPoints

        Parameters
        ----------
        faces
            A sequence of Face objects specifying the faces to partition.
        sketchPlane
            A planar Face or DatumPlane object.
        sketchPlaneSide
            A SymbolicConstant specifying the side of the plane to be used for sketching. Possible
            values are SIDE1 and SIDE2.
        sketchUpEdge
            An :py:class:`~abaqus.BasicGeometry.Edge.Edge` object specifying the orientation of **sketch**. This edge must not be orthogonal
            to **sketchPlane**.
        sketch
            A :py:class:`~abaqus.Sketcher.ConstrainedSketch.ConstrainedSketch` object specifying the partition.
        distance
            A Float specifying the projection distance. Possible values are **distance** >> 0.0.
        sketchOrientation
            A SymbolicConstant specifying the orientation of **sketchUpEdge** on the sketch. Possible
            values are RIGHT, LEFT, TOP, and BOTTOM. The default value is RIGHT.

        Returns
        -------
        Feature
            A :py:class:`~abaqus.Feature.Feature.Feature` object.

        Raises
        ------
        AbaqusException
        """
        ...

    @abaqus_method_doc
    def PartitionFaceBySketchRefPoint(
        self,
        faces: Tuple[Face, ...],
        sketchPlane: str,
        sketchUpEdge: Edge,
        sketch: ConstrainedSketch,
        point: int,
        sketchOrientation: Literal[RIGHT, LEFT, TOP, BOTTOM] = RIGHT,
    ) -> Feature:
        """This method partitions one or more faces by sketching on a sketch plane and then
        projecting the sketch toward the target faces through a distance governed by the
        reference point.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].parts[name].AttachmentPoints
                mdb.models[name].rootAssembly.AttachmentPoints

        Parameters
        ----------
        faces
            A sequence of Face objects specifying the faces to partition.
        sketchPlane
            A planar Face or DatumPlane object.
        sketchUpEdge
            An :py:class:`~abaqus.BasicGeometry.Edge.Edge` object or a DatumAxis object specifying the orientation of **sketch**. This edge
            or datum axis must not be orthogonal to **sketchPlane**.
        sketch
            A :py:class:`~abaqus.Sketcher.ConstrainedSketch.ConstrainedSketch` object specifying the partition.
        point
            A ConstrainedSketchVertex, InterestingPoint, or DatumPoint object specifying the distance to project
            **sketch**. The point must not lie on **sketchPlane**.
        sketchOrientation
            A SymbolicConstant specifying the orientation of **sketchUpEdge** on the sketch. Possible
            values are RIGHT, LEFT, TOP, and BOTTOM. The default value is RIGHT.

        Returns
        -------
        Feature
            A :py:class:`~abaqus.Feature.Feature.Feature` object.

        Raises
        ------
        AbaqusException
        """
        ...

    @abaqus_method_doc
    def PartitionFaceBySketchThruAll(
        self,
        faces: Tuple[Face, ...],
        sketchPlane: str,
        sketchPlaneSide: Literal[SIDE1, SIDE2],
        sketchUpEdge: str,
        sketch: ConstrainedSketch,
        sketchOrientation: Literal[RIGHT, LEFT, TOP, BOTTOM] = RIGHT,
    ) -> Feature:
        """This method partitions one or more faces by sketching on a sketch plane and then
        projecting toward the target faces through an infinite distance.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].parts[name].AttachmentPoints
                mdb.models[name].rootAssembly.AttachmentPoints

        Parameters
        ----------
        faces
            A sequence of Face objects specifying the faces to partition.
        sketchPlane
            A planar Face or DatumPlane object.
        sketchPlaneSide
            A SymbolicConstant specifying the extrude direction of the sketch. Possible values are
            SIDE1 and SIDE2.
        sketchUpEdge
            An Edge or a DatumAxis object specifying the orientation of **sketch**. This edge or datum
            axis must not be orthogonal to **sketchPlane**.
        sketch
            A :py:class:`~abaqus.Sketcher.ConstrainedSketch.ConstrainedSketch` object specifying the partition.
        sketchOrientation
            A SymbolicConstant specifying the orientation of **sketchUpEdge** on the sketch. Possible
            values are RIGHT, LEFT, TOP, and BOTTOM. The default value is RIGHT.

        Returns
        -------
        Feature
            A :py:class:`~abaqus.Feature.Feature.Feature` object.

        Raises
        ------
        AbaqusException
        """
        ...

    @abaqus_method_doc
    def ReferencePoint(
        self,
        point: Union[tuple, Vertex, InterestingPoint, MeshNode, Datum],
        instanceName: str = "",
    ) -> Feature:
        """This method creates a Feature object and a ReferencePoint object at the specified
        location.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].parts[name].AttachmentPoints
                mdb.models[name].rootAssembly.AttachmentPoints

        Parameters
        ----------
        point
            A ConstrainedSketchVertex, InterestingPoint, a MeshNode, or a Datum object specifying a reference point.
            **point** can also be a sequence of three Floats representing the **X**-, **Y**-, and
            **Z**-coordinates of the point.
        instanceName
            Used internally by the input file writer.

        Returns
        -------
        feature: Feature
            A :py:class:`~abaqus.Feature.Feature.Feature` object
        """
        return self

    @abaqus_method_doc
    def RemoveWireEdges(self, wireEdgeList: Tuple[Edge, ...]) -> Feature:
        """This method removes wire edges.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].parts[name].AttachmentPoints
                mdb.models[name].rootAssembly.AttachmentPoints

        Parameters
        ----------
        wireEdgeList
            A sequence of Edge objects specifying the edges to remove. Any specified edge that is
            not a wire edge will not be removed.

        Returns
        -------
        feature: Feature
            A :py:class:`~abaqus.Feature.Feature.Feature` object
        """
        ...

    @abaqus_method_doc
    def WirePolyLine(
        self,
        points: float,
        mergeType: Literal[MERGE, IMPRINT, SEPARATE] = IMPRINT,
        meshable: Boolean = ON,
    ) -> Feature:
        """This method creates an additional Feature object by creating a series of wires joining
        points in pairs. When such a feature is created at the Part level, then each point can
        be either a datum point, a vertex, a reference point, an interesting point, an orphan
        mesh node, or the coordinates of a point. When such a feature is created at the Assembly
        level, then each point can only be a vertex, a reference point, or an orphan mesh node.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].parts[name].AttachmentPoints
                mdb.models[name].rootAssembly.AttachmentPoints

        Parameters
        ----------
        points
            A tuple of point pairs, each pair being itself represented by a tuple. For part level
            features each point can be a ConstrainedSketchVertex, Datum point, Reference point, orphan mesh Node, or
            InterestingPoint object specifying the points through which the polyline wire will pass.
            Each point can also be a tuple of Floats representing the coordinates of a point. For
            assembly level features each point can only be a ConstrainedSketchVertex, Reference point, or orphan mesh
            Node specifying the points through which the polyline wire will pass (coordinates cannot
            be specified). In any of the pairs, the first or second point can be NONE. In that case,
            the point pair will create a zero-length wire, which is required for certain types of
            connectors. You must specify at least one pair.
        mergeType
            A SymbolicConstant specifying the merge behavior of the wire with existing geometry. If
            **mergeType** is MERGE, Abaqus merges the wire into solid regions of the part if the wire
            passes through them. If **mergeType** is IMPRINT, Abaqus imprints the wire on existing
            geometry as edges. If **mergeType** is SEPARATE, Abaqus neither merges nor imprints the
            spline wire with existing geometry. It creates the wire separately. The default value is
            IMPRINT.
        meshable
            A Boolean specifying whether the wire should be available for selection for meshing
            operations. If **meshable** = OFF, the wire can be used for connector section assignment.
            The default value is ON.

        Returns
        -------
        feature: Feature
            A :py:class:`~abaqus.Feature.Feature.Feature` object
        """
        ...

    @abaqus_method_doc
    def isSuppressed(self) -> bool:
        """This method queries the suppressed state of the feature.

        Returns
        -------
        Boolean
            A Boolean value of True if the feature is suppressed and False if it is not suppressed.
        """
        ...

    @abaqus_method_doc
    def restore(self) -> None:
        """This method restores the parameters of a feature to the value they had when the backup
        method was invoked on the part or assembly. Use the restore method after the backup
        method.
        """
        ...

    @abaqus_method_doc
    def resume(self) -> None:
        """This method resumes suppressed features. Resuming a feature fully restores it to the
        part or assembly. You can resume the last feature you suppressed, all suppressed
        features, or just selected features. When you resume a child feature, Abaqus/CAE also
        resumes the parent features automatically.
        """
        ...

    @abaqus_method_doc
    def setValues(
        self,
        parameter: float = ...,
        parameter1: float = ...,
        parameter2: float = ...,
        sketch: ConstrainedSketch = ...,
        distance: float = ...,
    ) -> None:
        """This method modifies the Feature object.

        Parameters
        ----------
        parameter
            A Float specifying the normalized distance along **edge** at which to partition. Possible
            values are 0.0 << **parameter** << 1.0. You use this argument to modify a partition
            created with the created with the PartitionEdgeByParam method.
        parameter1
            A Float specifying the distance along **edge1** at which to partition. Possible values are
            0.0 ≤ **parameter1** ≤ 1.0. You use this argument to modify a partition object created
            with the PartitionFaceByCurvedPathEdgeParam method.
        parameter2
            A Float specifying the distance along **edge2** at which to partition. Possible values are
            0.0 ≤ **parameter2** ≤ 1.0. You use this argument to modify a partition object created
            with the PartitionFaceByCurvedPathEdgeParam method.
        sketch
            A :py:class:`~abaqus.Sketcher.ConstrainedSketch.ConstrainedSketch` object specifying the partition. You use this argument to modify a
            partition object created with a sketch; for example, using the PartitionFaceBySketch
            method.
        distance
            A Float specifying the projection **distance**. Possible values are **distance** >> 0.0. You
            use this argument to modify a partition object created with the
            PartitionFaceBySketchDistance method.

        Raises
        ------
        AbaqusException
        """
        ...

    @abaqus_method_doc
    def suppress(self) -> None:
        """This method suppresses features. Suppressing a feature is equivalent to temporarily
        removing the feature from the part or assembly. Suppressed features remain suppressed
        when you regenerate a part or assembly. You cannot suppress the base feature. In
        addition, if you suppress a parent feature, all of its child features are also
        suppressed automatically. Suppressed features can be restored with the resume command.
        """
        ...



from abqpy.decorators import abaqus_class_doc


@abaqus_class_doc
class IgnoredVertex:
    """An :py:class:`~abaqus.BasicGeometry.IgnoredVertex.IgnoredVertex` object is a point region of the geometry that was abstracted away by a
    virtual topology feature.

    .. note:: 
        This object can be accessed by::

            import part
            mdb.models[name].parts[name].ignoredVertices[i]
            import assembly
            mdb.models[name].rootAssembly.allInstances[name].ignoredVertices[i]
            mdb.models[name].rootAssembly.instances[name].ignoredVertices[i]
    """

    #: An Int specifying the index of the IgnoredVertex in the IgnoredVertexArray.
    index: int = None

    #: A tuple of Floats specifying the **X**-, **Y**-, and **Z**-coordinates of the vertex.
    pointOn: float = None



import typing

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc
from ..Adaptivity.AdaptivityProcess import AdaptivityProcess
from ..Annotation.Annotation import Annotation
from ..CustomKernel.RepositorySupport import RepositorySupport
from ..EditMesh.MeshEditOptions import MeshEditOptions
from ..Job.Job import Job
from ..Job.OptimizationProcess import OptimizationProcess
from ..Model.Model import Model


@abaqus_class_doc
class MdbBase:
    """The Mdb object is the high-level Abaqus model database. A model database stores models
    and analysis controls.

    .. note:: 
        This object can be accessed by::

            mdb
    """

    #: An Int specifying the release number of the Mdb object in memory.
    version: int = None

    #: A Float specifying the value of a counter associated with the Mdb object. The counter
    #: indicates when the Mdb object was last changed.
    lastChangedCount: float = None

    #: A repository of Job objects.
    jobs: typing.Dict[str, Job] = {}

    #: A repository of AdaptivityProcess objects.
    adaptivityProcesses: typing.Dict[str, AdaptivityProcess] = {}

    #: A repository of OptimizationProcess objects.
    optimizationProcesses: typing.Dict[str, OptimizationProcess] = {}

    #: A :py:class:`~abaqus.EditMesh.MeshEditOptions.MeshEditOptions` object specifying the undo/redo behavior when editing meshes on parts
    #: or part instances.
    meshEditOptions: MeshEditOptions = MeshEditOptions()

    #: A repository of Model objects.
    models: typing.Dict[str, Model] = {}

    #: A :py:class:`~abaqus.CustomKernel.RepositorySupport.RepositorySupport` object.
    customData: RepositorySupport = RepositorySupport()

    #: A repository of Annotation objects.
    annotations: typing.Dict[str, Annotation] = {}

    @abaqus_method_doc
    def __init__(self, pathName: str = ""):
        """This constructor creates an empty Mdb object.

        .. note:: 
            This function can be accessed by::

                Mdb

        Parameters
        ----------
        pathName: str
            A String specifying the path to be used when the model database is saved to a file. If
            you do not provide a file extension, .cae is appended automatically to the path. The
            default value is an empty string.

        Returns
        -------
        Mdb
            A :py:class:`~abaqus.Mdb.Mdb.Mdb` object
        """
        self.pathName = pathName
        self.models["Model-1"] = Model("Model-1")
        self.models["Model-1"].FieldOutputRequest("F-Output-1", "Initial")

    @abaqus_method_doc
    def importDxf(self, fileName: str):
        """This method creates a ConstrainedSketch object from a file containing dxf-format
        (AutoCAD) geometry. Only a limited number of entities are supported. This format should
        be used only if no other formats are available.

        .. note:: 
            This function can be accessed by::

                Mdb

        Parameters
        ----------
        fileName
            A String specifying the path to the dxf file to open.

        Returns
        -------
        Mdb
            A :py:class:`~abaqus.Mdb.Mdb.Mdb` object
        """
        ...

    @abaqus_method_doc
    def openMdb(self, pathName: str):
        """This method opens an existing model database file.

        .. note:: 
            This function can be accessed by::

                Mdb

        Parameters
        ----------
        pathName: str
            A String specifying the path to the model database file to open. If you do not provide a
            file extension, Abaqus/CAE attempts to open the file with .cae appended to the path.

        Returns
        -------
        Mdb
            A :py:class:`~abaqus.Mdb.Mdb.Mdb` object

        Raises
        ------
        MdbError
            invalid model database;
            If the file is an invalid model database
        MdbError
            incompatible release number, expected *<Abaqus release>*, got *<earlier or later Abaqus release>*;
            If the file contains a model database from an Abaqus release other than the Abaqus
            release you are currently running
        MdbError
            cannot open file; may be in use by another CAE session;
            If the model database file is already opened in write mode
        MdbError
            cannot open file;
            If the command fails to open the model database file for reasons not mentioned above
        """
        ...

    @abaqus_method_doc
    def close(self):
        """This method closes an open Mdb object but does not save the Mdb object to disk. After
        closing the Mdb object, this method creates a new unnamed empty Mdb object.
        """
        ...

    @abaqus_method_doc
    def save(self):
        """This method saves an Mdb object to disk at the location specified by **pathName**
        (*pathName* is a member of the Mdb object).

        Raises
        ------
        MdbError
            cannot save file: pathname member is empty;
            If **pathName** is empty
        MdbError
            “abaqus.cae” is an invalid CAE filename;
            If **pathName** is abaqus.cae

        Parameters
        ----------
        MdbError
            cannot save file;
            If the command fails to save the Mdb object to disk for reasons not mentioned above
        """
        ...

    @abaqus_method_doc
    def saveAs(self, pathName: str):
        """This method saves an Mdb object to disk at the specified location.

        Parameters
        ----------
        pathName: str
            A String specifying the path to be used when the model database is saved to a file. If
            you do not provide a file extension, .cae is appended automatically to the path.

        Raises
        ------
        MdbError
            “abaqus.cae” is an invalid CAE filename;
            If **pathName** is abaqus.cae
        MdbError
            cannot save file;
            If the command fails to save the Mdb object to disk for reasons not mentioned above
        """
        ...

    @abaqus_method_doc
    def openAuxMdb(self, pathName: str):
        """This method opens an auxiliary Mdb object on the disk at the specified location. This
        enables models from the auxiliary Mdb object to be copied into the current Mdb.

        Parameters
        ----------
        pathName: str
            A String specifying the path to the auxiliary Mdb which is to be opened. If you do not
            provide a file extension, .cae is appended automatically to the path.

        Raises
        ------
        MdbError
            invalid model database;
            If the file is an invalid model database
        MdbError
            incompatible release number;
            If the file contains a model database from an Abaqus release other than the Abaqus
            release you are currently running
        MdbError
            cannot open file;
            If the command fails to open the model database file for reasons not mentioned above
        """
        ...

    @abaqus_method_doc
    def closeAuxMdb(self):
        """This method closes the auxiliary Mdb which had been opened earlier using the openAuxMdb
        command.

        Raises
        ------

        Parameters
        ----------
        MdbError
            The auxiliary Mdb was not opened;
            If the auxiliary Mdb was not opened earlier
        """
        ...

    @abaqus_method_doc
    def getAuxMdbModelNames(self):
        """This method returns a list of model names present in the auxiliary Mdb which had been
        opened earlier using the openAuxMdb command.

        Returns
        -------
        typing.List[str]
            A list of model names present in the auxiliaryMdb

        Raises
        ------
        MdbError
            The auxiliary Mdb was not opened;
            If the auxiliary Mdb was not opened earlier
        """
        ...

    @abaqus_method_doc
    def copyAuxMdbModel(self, fromName: str, toName: str = ""):
        """This method copies a specified model from the auxiliary Mdb which had been opened
        earlier using the openAuxMdb command.

        Parameters
        ----------
        fromName
            A String specifying the model name in the auxiliary Mdb which is to be copied.
        toName
            A String specifying the name to be given to the model after it is copied into the Mdb.
            If this argument is not specified **toName** is assumed to be the same as **fromName**. If a
            model with name **toName** already exists in Mdb, it is overwritten.

        Raises
        ------
        MdbError
            The auxiliary Mdb was not opened;
            If the auxiliary Mdb was not opened earlier
        KeyError
            fromName does not exist;
            If the model fromName does not exist in the auxiliary Mdb
        """
        ...



import typing

from abqpy.decorators import abaqus_class_doc


@abaqus_class_doc
class DrawingArea:
    """The DrawingArea object specifies the location and size of the drawing area used for
    placement of viewports.

    .. note::
        This object can be accessed by::

            session.drawingArea
    """

    #: A Float specifying the width in millimeters.
    width: float = None

    #: A Float specifying the height in millimeters.
    height: float = None

    #: A pair of Floats specifying the coordinates of the bottom left hand corner in
    #: millimeters.
    origin: typing.Tuple[float, ...] = (0.0, 0.0)



from abqpy.decorators import abaqus_class_doc, abaqus_method_doc
from .ContactProperty import ContactProperty
from ..UtilityAndView.abaqusConstants import *


@abaqus_class_doc
class FilmConditionProp(ContactProperty):
    """The FilmConditionProp object is an interaction property that defines a film coefficient
    as a function of temperature and field variables.
    The FilmConditionProp object is derived from the InteractionProperty object.

    .. note:: 
        This object can be accessed by::

            import interaction
            mdb.models[name].interactionProperties[name]

        The corresponding analysis keywords are:

        - FILM PROPERTY
    """

    #: A String specifying the interaction property repository key.
    name: str

    #: A Boolean specifying whether the data depend on temperature. The default value is OFF.
    temperatureDependency: Boolean = OFF

    #: An Int specifying the number of field variable dependencies. The default value is 0.
    dependencies: int = 0

    #: A sequence of sequences of Floats specifying the following:
    #: 
    #: - The film coefficient, hh.
    #: - Temperature, if the data depend on temperature.
    #: - Value of the first field variable, if the data depend on field variables.
    #: - Value of the second field variable.
    #: - Etc.
    property: tuple = ()

    @abaqus_method_doc
    def __init__(
        self,
        name: str,
        temperatureDependency: Boolean = OFF,
        dependencies: int = 0,
        property: tuple = (),
    ):
        """This method creates a FilmConditionProp object.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].FilmConditionProp

        Parameters
        ----------
        name
            A String specifying the interaction property repository key.
        temperatureDependency
            A Boolean specifying whether the data depend on temperature. The default value is OFF.
        dependencies
            An Int specifying the number of field variable dependencies. The default value is 0.
        property
            A sequence of sequences of Floats specifying the following:
            
            - The film coefficient, hh.
            - Temperature, if the data depend on temperature.
            - Value of the first field variable, if the data depend on field variables.
            - Value of the second field variable.
            - Etc.

        Returns
        -------
        FilmConditionProp
            A :py:class:`~abaqus.Interaction.FilmConditionProp.FilmConditionProp` object.
        """
        super().__init__(name)

    @abaqus_method_doc
    def setValues(
        self,
        temperatureDependency: Boolean = OFF,
        dependencies: int = 0,
        property: tuple = (),
    ):
        """This method modifies the FilmConditionProp object.

        Parameters
        ----------
        temperatureDependency
            A Boolean specifying whether the data depend on temperature. The default value is OFF.
        dependencies
            An Int specifying the number of field variable dependencies. The default value is 0.
        property
            A sequence of sequences of Floats specifying the following:
            
            - The film coefficient, hh.
            - Temperature, if the data depend on temperature.
            - Value of the first field variable, if the data depend on field variables.
            - Value of the second field variable.
            - Etc.
        """
        ...



from abqpy.decorators import abaqus_class_doc, abaqus_method_doc
from ....UtilityAndView.abaqusConstants import *


@abaqus_class_doc
class PorousElastic:
    r"""The PorousElastic object specifies elastic material properties for porous materials.

    .. note:: 
        This object can be accessed by::

            import material
            mdb.models[name].materials[name].porousElastic
            import odbMaterial
            session.odbs[name].materials[name].porousElastic

        The table data for this object are:

        - If **shear** = :math:`G`, the table data specify the following:
        
            - The logarithmic bulk modulus, :math:`\kappa`. (Dimensionless.)
            - The shear modulus, :math:`G`.
            - The elastic tensile limit, :math:`p_{t}^{e l}`.
              (This value cannot be negative.)
            - Temperature, if the data depend on temperature.
            - Value of the first field variable, if the data depend on field variables.
            - Value of the second field variable.
            - Etc.
            
        - If **shear** = POISSON, the table data specify the following:
        
            - The logarithmic bulk modulus, :math:`\kappa`. (Dimensionless.)
            - The Poisson's ratio, :math:`\nu`.
            - The elastic tensile limit, :math:`p_{t}^{e l}`. (This value cannot be negative.)
            - Temperature, if the data depend on temperature.
            - Value of the first field variable, if the data depend on field variables.
            - Value of the second field variable.
            - Etc.

        The corresponding analysis keywords are:

        - POROUS ELASTIC
    """

    @abaqus_method_doc
    def __init__(
        self,
        table: tuple,
        shear: SymbolicConstant = POISSON,
        temperatureDependency: Boolean = OFF,
        dependencies: int = 0,
    ):
        """This method creates a PorousElastic object.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].materials[name].PorousElastic
                session.odbs[name].materials[name].PorousElastic

        Parameters
        ----------
        table
            A sequence of sequences of Floats specifying the items described below.
        shear
            A SymbolicConstant specifying the shear definition form. Possible values are G and
            POISSON. The default value is POISSON.
        temperatureDependency
            A Boolean specifying whether the data depend on temperature. The default value is OFF.
        dependencies
            An Int specifying the number of field variable dependencies. The default value is 0.

        Returns
        -------
        PorousElastic
            A :py:class:`~abaqus.Material.Elastic.Porous.PorousElastic.PorousElastic` object.

        Raises
        ------
        RangeError
        """
        ...

    @abaqus_method_doc
    def setValues(self, *args, **kwargs):
        """This method modifies the PorousElastic object.

        Raises
        ------
        RangeError
        """
        ...



from abqpy.decorators import abaqus_class_doc, abaqus_method_doc
from ..UtilityAndView.abaqusConstants import *
from .._OptionsBase import _OptionsBase


@abaqus_class_doc
class AnimationController(_OptionsBase):
    """The AnimationController object controls all object-based animation to be displayed in
    the viewports. The AnimationController object has no constructor. Abaqus creates the
    **animationController** member when it creates the Session object.

    .. note:: 
        This object can be accessed by::

            import animation
            session.viewports[name].animationController
    """

    #: A SymbolicConstant specifying the type of movie to play. Possible values are
    #: SCALE_FACTOR, HARMONIC, TIME_HISTORY, and NONE. The default value is NONE.
    animationType: SymbolicConstant = NONE

    #: A SymbolicConstant specifying the state of the animation controller. Possible values are
    #: STOP and PLAY. The default value is STOP.
    state: SymbolicConstant = STOP

    @abaqus_method_doc
    def play(self, duration: SymbolicConstant = UNLIMITED):
        """This method begins the animation.

        Parameters
        ----------
        duration
            The SymbolicConstant UNLIMITED or an Int specifying how many seconds to play the
            animation. The default value is UNLIMITED.

        Raises
        ------
        AnimationError: animationType not set
            If **animationType** = NONE:
        """
        # TODO: implement this method
        ...

    @abaqus_method_doc
    def stop(self):
        """This method stops the animation."""
        # TODO: implement this method
        ...

    @abaqus_method_doc
    def incrementFrame(self):
        """This method increments the animation frame."""
        # TODO: implement this method
        ...

    @abaqus_method_doc
    def decrementFrame(self):
        """This method decrements the animation frame."""
        # TODO: implement this method
        ...

    @abaqus_method_doc
    def showFrame(self, frame: int = None, value: float = None):
        """This method renders the specified frame of the animation.

        Parameters
        ----------
        frame
            An Int specifying the frame number.
        value
            A Float specifying the frame: for **animationType** = TIME_HISTORY the frame with the time
            nearest to this value, for **animationType** = HARMONIC the frame with the angle nearest to
            this value, for **animationType** = SCALE_FACTOR the frame with the scale value nearest to
            this value.
        """
        # TODO: implement this method
        ...

    @abaqus_method_doc
    def showFirstFrame(self):
        """This method renders the first frame of the animation."""
        # TODO: implement this method
        ...

    @abaqus_method_doc
    def showLastFrame(self):
        """This method renders the last frame of the animation."""
        # TODO: implement this method
        ...

    @abaqus_method_doc
    def setValues(self, animationType: SymbolicConstant = NONE):
        """This method modifies the AnimationController object.

        Parameters
        ----------
        animationType
            A SymbolicConstant specifying the type of movie to play. Possible values are
            SCALE_FACTOR, HARMONIC, TIME_HISTORY, and NONE. The default value is NONE.

        Raises
        ------
        RangeError
        """
        super().setValues(animationType=animationType)



from abqpy.decorators import abaqus_class_doc, abaqus_method_doc
from ..Datum.DatumCsys import DatumCsys
from ..Region.Set import Set
from ..UtilityAndView.abaqusConstants import *


@abaqus_class_doc
class ConnectorOrientation:
    """The ConnectorOrientation object is used to assign a connector orientation to a
    connector.

    .. note:: 
        This object can be accessed by::

            import assembly
            mdb.models[name].rootAssembly.connectorOrientations[i]
            import odbAccess
            session.odbs[name].rootAssembly.connectorOrientations[i]
    """

    region: Set
    localCsys1: DatumCsys = DatumCsys()
    axis1: SymbolicConstant = (AXIS_1,)
    angle1: float = 0
    orient2sameAs1: Boolean = ON
    localCsys2: DatumCsys = DatumCsys()
    axis2: SymbolicConstant = AXIS_1
    angle2: float = 0

    #: A :py:class:`~abaqus.Region.Set.Set` object specifying the region to which the orientation is assigned.
    region: Set

    #: A :py:class:`~abaqus.Datum.DatumCsys.DatumCsys` object specifying the local coordinate system of the first connector point.
    #: This value may be None, indicating the global coordinate system.
    localCsys1: DatumCsys = None

    #: A SymbolicConstant specifying the axis of a datum coordinate system about which an
    #: additional rotation is applied. Possible values are AXIS_1, AXIS_2, and AXIS_3. The
    #: default value is AXIS_1.
    axis1: SymbolicConstant = AXIS_1

    #: A Float specifying the angle of the additional rotation. The default value is 0.0.
    angle1: float = 0

    #: A Boolean specifying whether or not the second connector point is to use the same local
    #: coordinate system, axis, and angle as the first point. The default value is ON.
    orient2sameAs1: Boolean = ON

    #: A :py:class:`~abaqus.Datum.DatumCsys.DatumCsys` object specifying the local coordinate system of the second connector point.
    #: This value may be None, indicating the global coordinate system.
    localCsys2: DatumCsys = None

    #: A SymbolicConstant specifying the axis of a datum coordinate system about which an
    #: additional rotation is applied. Possible values are AXIS_1, AXIS_2, and AXIS_3. The
    #: default value is AXIS_1.
    axis2: SymbolicConstant = AXIS_1

    #: A Float specifying the angle of the additional rotation. The default value is 0.0.
    angle2: float = 0

    @abaqus_method_doc
    def __init__(
        self,
        region: Set,
        localCsys1: DatumCsys = None,
        axis1: SymbolicConstant = AXIS_1,
        angle1: float = 0,
        orient2sameAs1: Boolean = ON,
        localCsys2: DatumCsys = None,
        axis2: SymbolicConstant = AXIS_1,
        angle2: float = 0,
    ):
        """This method creates a ConnectorOrientation object.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].rootAssembly.ConnectorOrientation
                session.odbs[name].rootAssembly.ConnectorOrientation

        Parameters
        ----------
        region
            A :py:class:`~abaqus.Region.Set.Set` object specifying the region to which the orientation is assigned.
        localCsys1
            A :py:class:`~abaqus.Datum.DatumCsys.DatumCsys` object specifying the local coordinate system of the first connector point.
            This value may be None, indicating the global coordinate system.
        axis1
            A SymbolicConstant specifying the axis of a datum coordinate system about which an
            additional rotation is applied. Possible values are AXIS_1, AXIS_2, and AXIS_3. The
            default value is AXIS_1.
        angle1
            A Float specifying the angle of the additional rotation. The default value is 0.0.
        orient2sameAs1
            A Boolean specifying whether or not the second connector point is to use the same local
            coordinate system, axis, and angle as the first point. The default value is ON.
        localCsys2
            A :py:class:`~abaqus.Datum.DatumCsys.DatumCsys` object specifying the local coordinate system of the second connector point.
            This value may be None, indicating the global coordinate system.
        axis2
            A SymbolicConstant specifying the axis of a datum coordinate system about which an
            additional rotation is applied. Possible values are AXIS_1, AXIS_2, and AXIS_3. The
            default value is AXIS_1.
        angle2
            A Float specifying the angle of the additional rotation. The default value is 0.0.

        Returns
        -------
        ConnectorOrientation
            A :py:class:`~abaqus.Assembly.ConnectorOrientation.ConnectorOrientation` object.
        """
        ...

    @abaqus_method_doc
    def setValues(self, *args, **kwargs):
        """This method modifies the ConnectorOrientation object."""
        ...



from abqpy.decorators import abaqus_class_doc, abaqus_method_doc

@abaqus_class_doc
class SimpleShearTestData:
    r"""The SimpleShearTestData object provides simple shear test data.

    .. note:: 
        This object can be accessed by::

            import material
            mdb.models[name].materials[name].hyperfoam.simpleShearTestData
            import odbMaterial
            session.odbs[name].materials[name].hyperfoam.simpleShearTestData

        The table data for this object are:

        - Nominal shear stress, :math:`T_{S}`.
        - Nominal shear strain, :math:`\gamma`
        - Nominal transverse stress, :math:`T_{T}` (normal to edge with shear stress). This stress value is optional.

        The corresponding analysis keywords are:

        - SIMPLE SHEAR TEST DATA
    """

    @abaqus_method_doc
    def __init__(self, table: tuple):
        """This method creates a SimpleShearTestData object.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].materials[name].hyperfoam.SimpleShearTestData
                session.odbs[name].materials[name].hyperfoam.SimpleShearTestData

        Parameters
        ----------
        table
            A sequence of sequences of Floats specifying the items described below.

        Returns
        -------
        SimpleShearTestData
            A :py:class:`~abaqus.Material.TestData.SimpleShearTestData.SimpleShearTestData` object.
        """
        ...

    @abaqus_method_doc
    def setValues(self, *args, **kwargs):
        """This method modifies the SimpleShearTestData object."""
        ...



import typing

from abqpy.decorators import abaqus_function_doc

"""These functions display status information. 

"""


@abaqus_function_doc
@typing.overload
def milestone(message: str):
    """This function displays a string in the prompt area.

    .. note:: 
        This function can be accessed by::

            milestone

    Parameters
    ----------
    message
        A String specifying the text to display.
    """
    ...


@abaqus_function_doc
@typing.overload
def milestone(message: str, percent: int):
    """This function displays a percentage complete message in the prompt area.

    .. note:: 
        This function can be accessed by::

            milestone

    Parameters
    ----------
    message
        A String specifying the text to display.
    percent
        An Int specifying the percentage complete.
    """
    ...


@abaqus_function_doc
@typing.overload
def milestone(message: str, object: str, done: int, total: int):
    """This function displays a message in the prompt area indicating the number done out of a
    total. The form of the message is `operation: object nn out of nn`

    .. note:: 
        This function can be accessed by::

            milestone

    Parameters
    ----------
    message
        A String specifying the operation.
    object
        A String specifying the object.
    done
        An Int specifying the number being processed.
    total
        An Int specifying the total number.
    """
    ...


@abaqus_function_doc
def milestone(*args, **kwargs):
    ...



from abqpy.decorators import abaqus_class_doc
from ..UtilityAndView.abaqusConstants import *


@abaqus_class_doc
class DGDisplayBodyOptions:
    """The DGDisplayBodyOptions object stores values and attributes applied to display bodies.
    The DGDisplayBodyOptions object has no constructor command. Abaqus creates an
    *odbDisplayOptions.displayBodyOptions* member when a display group instance is created,
    using values from *odbDisplay.displayBodyOptions*.

    .. note:: 
        This object can be accessed by::

            session.viewports[name].assemblyDisplay.displayGroupInstances[name].odbDisplayOptions.displayBodyOptions
            session.viewports[name].layers[name].assemblyDisplay.displayGroupInstances[name].odbDisplayOptions.displayBodyOptions
            session.viewports[name].layers[name].odbDisplay.displayGroupInstances[name].odbDisplayOptions.displayBodyOptions
            session.viewports[name].layers[name].partDisplay.displayGroupInstances[name].odbDisplayOptions.displayBodyOptions
            session.viewports[name].odbDisplay.displayGroupInstances[name].odbDisplayOptions.displayBodyOptions
            session.viewports[name].partDisplay.displayGroupInstances[name].odbDisplayOptions.displayBodyOptions
    """

    #: A SymbolicConstant specifying which edges to plot. Possible values are ALL, EXTERIOR,
    #: FEATURE, FREE, and NONE. The default value is EXTERIOR.NONE can be used only when
    #: **renderStyle** = SHADED.
    visibleEdges: SymbolicConstant = EXTERIOR

    #: A SymbolicConstant specifying the edge line style. Possible values are SOLID, DASHED,
    #: DOTTED, and DOT_DASH. The default value is SOLID.
    edgeLineStyle: SymbolicConstant = SOLID

    #: A SymbolicConstant specifying the edge line thickness. Possible values are VERY_THIN,
    #: THIN, MEDIUM, and THICK. The default value is VERY_THIN.
    edgeLineThickness: SymbolicConstant = VERY_THIN

    #: A Boolean specifying whether to allow color coded items in the output database to
    #: override the edge and fill color settings. The default value is ON.
    colorCodeOverride: Boolean = ON

    #: A Boolean specifying whether elements are displayed in a shrunk format. The default
    #: value is OFF.
    elementShrink: Boolean = OFF

    #: An Int specifying the percentage to shrink the elements when **elementShrink** = ON.
    #: Possible values are 0≤ **elementShrinkPercentage** ≤ 90. The default value is 5.
    elementShrinkFactor: int = 5

    #: A Boolean specifying whether to scale coordinates. The default value is OFF.
    coordinateScale: Boolean = OFF

    #: A Boolean specifying whether to set translucency. The default value is OFF.
    translucency: Boolean = OFF

    #: A Float specifying the translucency factor when **translucency** = ON. Possible values are
    #: 0.0≤ **translucencyFactor** ≤ 1.0. The default value is 0.3.
    translucencyFactor: float = 0

    #: A String specifying the color to be used to plot the edges of the model when
    #: **renderStyle** = WIREFRAME or HIDDEN. The default value is "White".
    edgeColorWireHide: str = ""

    #: A String specifying the color to be used to plot the edges of the model when
    #: **renderStyle** = FILLED or SHADED. The default value is "Black".
    edgeColorFillShade: str = ""

    #: A String specifying the color to be used to fill elements when **renderStyle** = FILLED or
    #: SHADED. The default value is "White".
    fillColor: str = ""

    #: A tuple of three Floats specifying the coordinate scaling in each of the three
    #: coordinate directions when **coordinateScale** = ON. The default value is (1, 1, 1).
    coordinateScaleFactors: float = None



from abqpy.decorators import abaqus_class_doc, abaqus_method_doc
from typing_extensions import Literal
from ..Model.ModelBase import ModelBase
from ..Part.Part import Part
from ..UtilityAndView.abaqusConstants import *


@abaqus_class_doc
class PartModel(ModelBase):
    """Abaqus creates a Model object named `Model-1` when a session is started.

    .. note:: 
        This object can be accessed by::

            mdb.models[name]
    """

    @abaqus_method_doc
    def Part(
        self,
        name: str,
        dimensionality: Literal[THREE_D, TWO_D_PLANAR, AXISYMMETRIC],
        type: Literal[
            DEFORMABLE_BODY, EULERIAN, DISCRETE_RIGID_SURFACE, ANALYTIC_RIGID_SURFACE
        ],
        twist: Boolean = OFF,
    ):
        """This method creates a Part object and places it in the parts repository.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].Part

        Parameters
        ----------
        name
            A String specifying the repository key.
        dimensionality
            A SymbolicConstant specifying the dimensionality of the part. Possible values are
            THREE_D, TWO_D_PLANAR, and AXISYMMETRIC.
        type
            A SymbolicConstant specifying the type of the part. Possible values are DEFORMABLE_BODY,
            EULERIAN, DISCRETE_RIGID_SURFACE, and ANALYTIC_RIGID_SURFACE.
        twist
            A Boolean specifying whether to include a twist DEGREE OF FREEDOM in the part (only
            available when **dimensionality** = AXISYMMETRIC and **type** = DEFORMABLE_BODY). The default
            value is OFF.

        Returns
        -------
        Part
            A :py:class:`~abaqus.Part.Part.Part` object.
        """
        self.parts[name] = part = Part(name, dimensionality, type, twist)
        return part



from __future__ import annotations
from typing import Union, List, Dict

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc
from .IgnoredEdge import IgnoredEdge
from ..UtilityAndView.abaqusConstants import *


@abaqus_class_doc
class IgnoredEdgeArray(List[IgnoredEdge]):
    """The IgnoredEdgeArray is a sequence of IgnoredEdge objects. If the part is modified, then
    IgnoredEdgeArray must be updated for that part.

    .. note:: 
        This object can be accessed by::

            import part
            mdb.models[name].parts[name].ignoredEdges
            import assembly
            mdb.models[name].rootAssembly.allInstances[name].ignoredEdges
            mdb.models[name].rootAssembly.instances[name].ignoredEdges
    """

    @abaqus_method_doc
    def findAt(self, coordinates: tuple, printWarning: Boolean = True) -> Union[IgnoredEdge, List[IgnoredEdge]]:
        """This method returns the object or objects in the IgnoredEdgeArray located at the given
        coordinates.
        findAt initially uses the ACIS tolerance of 1E-6. As a result, findAt returns any
        IgnoredEdge that is at the arbitrary point specified or at a distance of less than 1E-6
        from the arbitrary point. If nothing is found, findAt uses the tolerance for imprecise
        geometry (applicable only for imprecise geometric entities). The arbitrary point must
        not be shared by a second IgnoredEdge. If two IgnoredEdge objects intersect or coincide
        at the arbitrary point, findAt chooses the first IgnoredEdge that it encounters, and you
        should not rely on the return value being consistent.
        findAt will always try to find objects among all the ignored edges in the part or
        assembly instance and will not restrict itself to a subset even if the IgnoredEdgeArray
        represents such subset.

        Parameters
        ----------
        coordinates
            A sequence of Floats specifying the **X**-, **Y**-, and **Z**- coordinates of the object to
            find.findAt returns either an IgnoredEdge object or a sequence of IgnoredEdge objects
            based on the type of input.If **coordinates** is a sequence of Floats, findAt returns the
            IgnoredEdge object at that point.If you omit the **coordinates** keyword argument, findAt
            accepts as arguments a sequence of sequence of floats in the following
            format::
            
                ignoredEdges = e.findAt(((20.19686, -169.513997, 27.798593), ),
                                        ((19.657627, -167.295749, 27.056402), ),
                                        ((18.274129, -157.144741, 25.15218), ))
        printWarning
            A Boolean specifying whether a message is to be printed to the CLI if no entity is found
            at the specified location. The default value is True.

        Returns
        -------
        IgnoredEdge
            An :py:class:`~abaqus.BasicGeometry.IgnoredEdge.IgnoredEdge` object or a sequence of IgnoredEdge objects.

        """
        return IgnoredEdge()

    @abaqus_method_doc
    def getSequenceFromMask(self, mask: str):
        """This method returns the object or objects in the IgnoredEdgeArray identified using the
        specified **mask**. This command is generated when the JournalOptions are set to
        COMPRESSEDINDEX. When large number of objects are involved, this method is highly
        efficient.

        Parameters
        ----------
        mask
            A String specifying the object or objects.

        Returns
        -------
        IgnoredEdge
            An :py:class:`~abaqus.BasicGeometry.IgnoredEdge.IgnoredEdge` object or a sequence of IgnoredEdge objects.

        """
        ...

    @abaqus_method_doc
    def getMask(self):
        """This method returns a string specifying the object or objects.

        Returns
        -------
        str
            A String specifying the object or objects.

        """
        ...

    @abaqus_method_doc
    def getClosest(self, coordinates: tuple, searchTolerance: str = "") -> Dict:
        """This method returns a object or objects in the IgnoredEdgeArray closest to the given set
        of points, where the given points need not lie on the edges in the IgnoredEdgeArray.

        Parameters
        ----------
        coordinates
            A sequence of a sequence of floats, where each sequence of floats describes the **X**-,
            **Y**-, and **Z**-coordinates of a point::
            
                >>> r=e.getClosest(coordinates=((20.0, 20.0, 10.0),(-1.0, -15.0, 15), ))
                >>> r.keys()
                [0, 1]
                >>> r[0]
                (mdb.models['Model-1'].parts['Part-1'].ignoredEdges[3], (15.7090625762939, 20.0, 10.0))
        searchTolerance
            A double specifying the distance within which the closest object must lie. The default
            value is half of the parent part/instance size.

        Returns
        -------
        dict
            This method returns a dictionary object. The key to the dictionary object is the
            position of the input point in the tuple specified in the **coordinates** starting at
            index 0. If a closest IgnoredEdge could be found then the value is a sequence consisting
            of two objects. The first object in the sequence is an IgnoredEdge that is close to the
            input point referred to by the key. The second object in the sequence, is a sequence of
            floats which specify the **X**-, **Y**-, and **Z**- location of the closest point on the
            IgnoredEdge to the given point. See program listing above.

        Raises
        ------
        Error: The mask results in an empty sequence
            An exception occurs if the resulting sequence is empty.
        """
        ...



import typing

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc
from .AdaptiveMeshConstraint import AdaptiveMeshConstraint
from ..Region.Region import Region
from ..UtilityAndView.abaqusConstants import *


@abaqus_class_doc
class DisplacementAdaptiveMeshConstraint(AdaptiveMeshConstraint):
    """The AdaptivityProcess object defines a series of jobs that will be submitted for
    analysis. Abaqus performs adaptive remeshing between each job.

    .. note:: 
        This object can be accessed by::

            import step
            mdb.models[name].adaptiveMeshConstraints[name]
    """

    #: A String specifying the adaptive mesh constraint repository key.
    name: str = ""

    #: A SymbolicConstant specifying the category of the adaptive mesh constraint. Possible
    #: values are MECHANICAL and THERMAL.
    category: SymbolicConstant = None

    #: A :py:class:`~abaqus.Region.Region.Region` object specifying the region to which the adaptive mesh constraint is applied.
    region: Region = Region()

    #: None or a DatumCsys object specifying the local coordinate system of the adaptive mesh
    #: constraint's degrees of freedom. If **localCsys** = None, the degrees of freedom are defined
    #: in the global coordinate system. The default value is None.
    localCsys: str = None

    @abaqus_method_doc
    def __init__(
        self,
        name: str,
        createStepName: str,
        region: Region,
        u1: typing.Union[SymbolicConstant, float] = UNSET,
        u2: typing.Union[SymbolicConstant, float] = UNSET,
        u3: typing.Union[SymbolicConstant, float] = UNSET,
        ur1: typing.Union[SymbolicConstant, float] = UNSET,
        ur2: typing.Union[SymbolicConstant, float] = UNSET,
        ur3: typing.Union[SymbolicConstant, float] = UNSET,
        amplitude: str = UNSET,
        motionType: SymbolicConstant = INDEPENDENT,
        localCsys: str = None,
    ):
        """This method creates a DisplacementAdaptiveMeshConstraint object.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].DisplacementAdaptiveMeshConstraint

        Parameters
        ----------
        name
            A String specifying the adaptive mesh constraint repository key.
        createStepName
            A String specifying the name of the step in which the adaptive mesh constraint is
            created.
        region
            A :py:class:`~abaqus.Region.Region.Region` object specifying the region to which the adaptive mesh constraint is applied.
        u1
            A Float or a SymbolicConstant specifying the displacement component in the 1-direction.
            Possible values for the SymbolicConstant are UNSET and SET. The default value is
            UNSET.Note:Although **u1**, **u2**, **u3**, **ur1**, **ur2**, and **ur3** are optional arguments, at
            least one of them must be specified.
        u2
            A Float or a SymbolicConstant specifying the displacement component in the 2-direction.
            Possible values for the SymbolicConstant are UNSET and SET. The default value is UNSET.
        u3
            A Float or a SymbolicConstant specifying the displacement component in the 3-direction.
            Possible values for the SymbolicConstant are UNSET and SET. The default value is UNSET.
        ur1
            A Float or a SymbolicConstant specifying the rotational displacement component about the
            1-direction. Possible values for the SymbolicConstant are UNSET and SET. The default
            value is UNSET.
        ur2
            A Float or a SymbolicConstant specifying the rotational displacement component about the
            2-direction. Possible values for the SymbolicConstant are UNSET and SET. The default
            value is UNSET.
        ur3
            A Float or a SymbolicConstant specifying the rotational displacement component about the
            3-direction. Possible values for the SymbolicConstant are UNSET and SET. The default
            value is UNSET.
        amplitude
            A String or the SymbolicConstant UNSET specifying the name of the amplitude reference.
            UNSET should be used if the adaptive mesh constraint has no amplitude reference. The
            default value is UNSET. You should provide the **amplitude** argument only if it is valid
            for the specified step.
        motionType
            A SymbolicConstant specifying the mesh motion in relation to the underlying material.
            Possible values are INDEPENDENT, FOLLOW and USER_DEFINED. The default value is
            INDEPENDENT.
        localCsys
            None or a DatumCsys object specifying the local coordinate system of the adaptive mesh
            constraint's degrees of freedom. If **localCsys** = None, the degrees of freedom are defined
            in the global coordinate system. The default value is None.

        Returns
        -------
        DisplacementAdaptiveMeshConstraint
            A :py:class:`~abaqus.Adaptivity.DisplacementAdaptiveMeshConstraint.DisplacementAdaptiveMeshConstraint` object.
        """
        super().__init__(name, region=region, category=MECHANICAL, localCsys=localCsys)

    @abaqus_method_doc
    def setValues(
        self,
        u1: typing.Union[SymbolicConstant, float] = UNSET,
        u2: typing.Union[SymbolicConstant, float] = UNSET,
        u3: typing.Union[SymbolicConstant, float] = UNSET,
        ur1: typing.Union[SymbolicConstant, float] = UNSET,
        ur2: typing.Union[SymbolicConstant, float] = UNSET,
        ur3: typing.Union[SymbolicConstant, float] = UNSET,
        amplitude: str = UNSET,
        motionType: SymbolicConstant = INDEPENDENT,
        localCsys: str = None,
    ):
        """This method modifies the data for an existing DisplacementAdaptiveMeshConstraint object
        in the step where it is created.

        Parameters
        ----------
        u1
            A Float or a SymbolicConstant specifying the displacement component in the 1-direction.
            Possible values for the SymbolicConstant are UNSET and SET. The default value is
            UNSET.Note:Although **u1**, **u2**, **u3**, **ur1**, **ur2**, and **ur3** are optional arguments, at
            least one of them must be specified.
        u2
            A Float or a SymbolicConstant specifying the displacement component in the 2-direction.
            Possible values for the SymbolicConstant are UNSET and SET. The default value is UNSET.
        u3
            A Float or a SymbolicConstant specifying the displacement component in the 3-direction.
            Possible values for the SymbolicConstant are UNSET and SET. The default value is UNSET.
        ur1
            A Float or a SymbolicConstant specifying the rotational displacement component about the
            1-direction. Possible values for the SymbolicConstant are UNSET and SET. The default
            value is UNSET.
        ur2
            A Float or a SymbolicConstant specifying the rotational displacement component about the
            2-direction. Possible values for the SymbolicConstant are UNSET and SET. The default
            value is UNSET.
        ur3
            A Float or a SymbolicConstant specifying the rotational displacement component about the
            3-direction. Possible values for the SymbolicConstant are UNSET and SET. The default
            value is UNSET.
        amplitude
            A String or the SymbolicConstant UNSET specifying the name of the amplitude reference.
            UNSET should be used if the adaptive mesh constraint has no amplitude reference. The
            default value is UNSET. You should provide the **amplitude** argument only if it is valid
            for the specified step.
        motionType
            A SymbolicConstant specifying the mesh motion in relation to the underlying material.
            Possible values are INDEPENDENT, FOLLOW and USER_DEFINED. The default value is
            INDEPENDENT.
        localCsys
            None or a DatumCsys object specifying the local coordinate system of the adaptive mesh
            constraint's degrees of freedom. If **localCsys** = None, the degrees of freedom are defined
            in the global coordinate system. The default value is None.
        """
        ...

    @abaqus_method_doc
    def setValuesInStep(
        self,
        stepName: str,
        u1: typing.Union[SymbolicConstant, float] = None,
        u2: typing.Union[SymbolicConstant, float] = None,
        u3: typing.Union[SymbolicConstant, float] = None,
        ur1: typing.Union[SymbolicConstant, float] = None,
        ur2: typing.Union[SymbolicConstant, float] = None,
        ur3: typing.Union[SymbolicConstant, float] = None,
        amplitude: str = "",
    ):
        """This method modifies the propagating data for an existing
        DisplacementAdaptiveMeshConstraint object in the specified step.

        Parameters
        ----------
        stepName
            A String specifying the name of the step in which the adaptive mesh constraint is
            modified.
        u1
            A Float or a SymbolicConstant specifying the displacement component in the 1-direction.
            Possible values for the SymbolicConstant are SET, UNCHANGED, and FREED.
        u2
            A Float or a SymbolicConstant specifying the displacement component in the 2-direction.
            Possible values for the SymbolicConstant are SET, UNCHANGED, and FREED.
        u3
            A Float or a SymbolicConstant specifying the displacement component in the 3-direction.
            Possible values for the SymbolicConstant are SET, UNCHANGED, and FREED.
        ur1
            A Float or a SymbolicConstant specifying the rotational displacement component about the
            1-direction. Possible values for the SymbolicConstant are SET, UNCHANGED, and FREED.
        ur2
            A Float or a SymbolicConstant specifying the rotational displacement component about the
            2-direction. Possible values for the SymbolicConstant are SET, UNCHANGED, and FREED.
        ur3
            A Float or a SymbolicConstant specifying the rotational displacement component about the
            3-direction. Possible values for the SymbolicConstant are SET, UNCHANGED, and FREED.
        amplitude
            A String or a SymbolicConstant specifying the name of the amplitude reference. Possible
            values for the SymbolicConstant are UNCHANGED and FREED. UNCHANGED should be used if the
            amplitude is propagated from the previous analysis step. FREED should be used if the
            adaptive mesh constraint is changed to have no amplitude reference. You should provide
            the **amplitude** argument only if it is valid for the specified step.
        """
        ...



from abqpy.decorators import abaqus_class_doc
from .LoadState import LoadState
from ..UtilityAndView.abaqusConstants import *


@abaqus_class_doc
class MomentState(LoadState):
    """The MomentState object stores the propagating data for a moment in a step. One instance
    of this object is created internally by the Moment object for each step. The instance is
    also deleted internally by the Moment object.
    The MomentState object has no constructor or methods.
    The MomentState object is derived from the LoadState object.

    .. note:: 
        This object can be accessed by::

            import load
            mdb.models[name].steps[name].loadStates[name]

        The corresponding analysis keywords are:

        - CLOAD
    """

    #: A Float or a Complex specifying the load component in the 4-direction.
    cm1: float = None

    #: A Float or a Complex specifying the load component in the 5-direction.
    cm2: float = None

    #: A Float or a Complex specifying the load component in the 6-direction.
    cm3: float = None

    #: A SymbolicConstant specifying the propagation state of the load component in the
    #: 4-direction. Possible values are UNSET, SET, UNCHANGED, and FREED.
    cm1State: SymbolicConstant = None

    #: A SymbolicConstant specifying the propagation state of the load component in the
    #: 5-direction. Possible values are UNSET, SET, UNCHANGED, and FREED.
    cm2State: SymbolicConstant = None

    #: A SymbolicConstant specifying the propagation state of the load component in the
    #: 6-direction. Possible values are UNSET, SET, UNCHANGED, and FREED.
    cm3State: SymbolicConstant = None

    #: A SymbolicConstant specifying the propagation state of the **amplitude** member. Possible
    #: values are UNSET, SET, UNCHANGED, and FREED.
    amplitudeState: SymbolicConstant = None

    #: A SymbolicConstant specifying the propagation state of the LoadState object. Possible
    #: values are:
    #: 
    #: - NOT_YET_ACTIVE
    #: - CREATED
    #: - PROPAGATED
    #: - MODIFIED
    #: - DEACTIVATED
    #: - NO_LONGER_ACTIVE
    #: - TYPE_NOT_APPLICABLE
    #: - INSTANCE_NOT_APPLICABLE
    #: - BUILT_INTO_BASE_STATE
    status: SymbolicConstant = None

    #: A String specifying the name of the amplitude reference. The String is empty if the load
    #: has no amplitude reference.
    amplitude: str = ""
